@@ -5,11 +5,7 @@ const { throws, doesNotThrow } = require('assert');
 
 const bench = common.createBenchmark(main, {
   n: [1e4],
-  method: [
-    'doesNotThrow',
-    'throws_TypeError',
-    'throws_RegExp'
-  ]
+  method: [ 'doesNotThrow', 'throws_TypeError', 'throws_RegExp' ],
 });
 
 function main({ n, method }) {
@@ -17,28 +13,25 @@ function main({ n, method }) {
   const doNotThrowError = () => { return 'foobar'; };
   const regExp = /foobar/;
   const message = 'failure';
-  var i;
 
   switch (method) {
-    case '':
-      // Empty string falls through to next line as default, mostly for tests.
     case 'doesNotThrow':
       bench.start();
-      for (i = 0; i < n; ++i) {
+      for (let i = 0; i < n; ++i) {
         doesNotThrow(doNotThrowError);
       }
       bench.end(n);
       break;
     case 'throws_TypeError':
       bench.start();
-      for (i = 0; i < n; ++i) {
+      for (let i = 0; i < n; ++i) {
         throws(throwError, TypeError, message);
       }
       bench.end(n);
       break;
     case 'throws_RegExp':
       bench.start();
-      for (i = 0; i < n; ++i) {
+      for (let i = 0; i < n; ++i) {
         throws(throwError, regExp, message);
       }
       bench.end(n);
