@@ -25,6 +25,7 @@
 #include "clocks.h"
 #include "path_resolver.h"
 #include "poll_oneoff.h"
+#include "sync_helpers.h"
 #include "wasi_rights.h"
 #include "wasi_serdes.h"
 #include "debug.h"
@@ -231,6 +232,13 @@ static uvwasi_errno_t uvwasi__setup_ciovs(const uvwasi_t* uvwasi,
   return UVWASI_ESUCCESS;
 }
 
+typedef struct new_connection_data_s {
+  int done;
+} new_connection_data_t;
+
+void on_new_connection(uv_stream_t *server, int status) {
+  // just do nothing
+}
 
 uvwasi_errno_t uvwasi_init(uvwasi_t* uvwasi, const uvwasi_options_t* options) {
   uv_fs_t realpath_req;
@@ -243,11 +251,16 @@ uvwasi_errno_t uvwasi_init(uvwasi_t* uvwasi, const uvwasi_options_t* options) {
   uvwasi_size_t env_buf_size;
   uvwasi_size_t i;
   int r;
+  struct sockaddr_in addr;
 
   if (uvwasi == NULL || options == NULL || options->fd_table_size == 0)
     return UVWASI_EINVAL;
 
+  // loop is only needed if there were pre-open sockets
+  uvwasi->loop = NULL;
+
   uvwasi->allocator = options->allocator;
+
   if (uvwasi->allocator == NULL)
     uvwasi->allocator = &default_allocator;
 
@@ -328,6 +341,14 @@ uvwasi_errno_t uvwasi_init(uvwasi_t* uvwasi, const uvwasi_options_t* options) {
     }
   }
 
+  for (i = 0; i < options->preopen_socketc; ++i) {
+    if (options->preopen_sockets[i].address == NULL ||
+        options->preopen_sockets[i].port > 65535) {
+      err = UVWASI_EINVAL;
+      goto exit;
+    }
+  }
+
   err = uvwasi_fd_table_init(uvwasi, options);
   if (err != UVWASI_ESUCCESS)
     goto exit;
@@ -363,6 +384,36 @@ uvwasi_errno_t uvwasi_init(uvwasi_t* uvwasi, const uvwasi_options_t* options) {
       goto exit;
   }
 
+  if (options->preopen_socketc > 0) {
+    uvwasi->loop = uvwasi__malloc(uvwasi, sizeof(uv_loop_t));
+    r = uv_loop_init(uvwasi->loop);
+    if (r != 0) {
+      err = uvwasi__translate_uv_error(r);
+      goto exit;
+    }
+  }
+
+  for (i = 0; i < options->preopen_socketc; ++i) {
+    uv_tcp_t* socket = (uv_tcp_t*) malloc(sizeof(uv_tcp_t));
+    uv_tcp_init(uvwasi->loop, socket);
+
+    uv_ip4_addr(options->preopen_sockets[i].address, options->preopen_sockets[i].port, &addr);
+
+    uv_tcp_bind(socket, (const struct sockaddr*)&addr, 0);
+    r = uv_listen((uv_stream_t*) socket, 128, on_new_connection);
+    if (r != 0) {
+      err = uvwasi__translate_uv_error(r);
+      goto exit;
+    }
+
+    err = uvwasi_fd_table_insert_preopen_socket(uvwasi,
+                                         uvwasi->fds,
+                                         socket);
+
+    if (err != UVWASI_ESUCCESS)
+      goto exit;
+  }
+
   return UVWASI_ESUCCESS;
 
 exit:
@@ -380,6 +431,12 @@ void uvwasi_destroy(uvwasi_t* uvwasi) {
   uvwasi__free(uvwasi, uvwasi->argv);
   uvwasi__free(uvwasi, uvwasi->env_buf);
   uvwasi__free(uvwasi, uvwasi->env);
+  if (uvwasi->loop != NULL) {
+    uv_stop(uvwasi->loop);
+    uv_loop_close(uvwasi->loop);
+    uvwasi__free(uvwasi, uvwasi->loop);
+    uvwasi->loop = NULL;
+  }
   uvwasi->fds = NULL;
   uvwasi->argv_buf = NULL;
   uvwasi->argv = NULL;
@@ -401,6 +458,8 @@ void uvwasi_options_init(uvwasi_options_t* options) {
   options->envp = NULL;
   options->preopenc = 0;
   options->preopens = NULL;
+  options->preopen_socketc = 0;
+  options->preopen_sockets = NULL;
   options->allocator = NULL;
 }
 
@@ -694,10 +753,9 @@ exit:
   return err;
 }
 
-
 uvwasi_errno_t uvwasi_fd_close(uvwasi_t* uvwasi, uvwasi_fd_t fd) {
   struct uvwasi_fd_wrap_t* wrap;
-  uvwasi_errno_t err;
+  uvwasi_errno_t err = 0;
   uv_fs_t req;
   int r;
 
@@ -712,9 +770,18 @@ uvwasi_errno_t uvwasi_fd_close(uvwasi_t* uvwasi, uvwasi_fd_t fd) {
   if (err != UVWASI_ESUCCESS)
     goto exit;
 
-  r = uv_fs_close(NULL, &req, wrap->fd, NULL);
-  uv_mutex_unlock(&wrap->mutex);
-  uv_fs_req_cleanup(&req);
+  if (wrap->sock == NULL) {
+    r = uv_fs_close(NULL, &req, wrap->fd, NULL);
+    uv_mutex_unlock(&wrap->mutex);
+    uv_fs_req_cleanup(&req);
+  } else {
+    r = 0;
+    err = free_handle_sync(uvwasi, (uv_handle_t*) wrap->sock);
+    uv_mutex_unlock(&wrap->mutex);
+    if (err != UVWASI_ESUCCESS) {
+      goto exit;
+    }   
+  }
 
   if (r != 0) {
     err = uvwasi__translate_uv_error(r);
@@ -1122,7 +1189,7 @@ uvwasi_errno_t uvwasi_fd_prestat_get(uvwasi_t* uvwasi,
   }
 
   buf->pr_type = UVWASI_PREOPENTYPE_DIR;
-  buf->u.dir.pr_name_len = strlen(wrap->path) + 1;
+  buf->u.dir.pr_name_len = strlen(wrap->path);
   err = UVWASI_ESUCCESS;
 exit:
   uv_mutex_unlock(&wrap->mutex);
@@ -1156,7 +1223,7 @@ uvwasi_errno_t uvwasi_fd_prestat_dir_name(uvwasi_t* uvwasi,
     goto exit;
   }
 
-  size = strlen(wrap->path) + 1;
+  size = strlen(wrap->path);
   if (size > (size_t) path_len) {
     err = UVWASI_ENOBUFS;
     goto exit;
@@ -1997,6 +2064,7 @@ uvwasi_errno_t uvwasi_path_open(uvwasi_t* uvwasi,
   err = uvwasi_fd_table_insert(uvwasi,
                                uvwasi->fds,
                                r,
+                               NULL,
                                resolved_path,
                                resolved_path,
                                filetype,
@@ -2520,7 +2588,6 @@ uvwasi_errno_t uvwasi_sched_yield(uvwasi_t* uvwasi) {
   return UVWASI_ESUCCESS;
 }
 
-
 uvwasi_errno_t uvwasi_sock_recv(uvwasi_t* uvwasi,
                                 uvwasi_fd_t sock,
                                 const uvwasi_iovec_t* ri_data,
@@ -2528,10 +2595,50 @@ uvwasi_errno_t uvwasi_sock_recv(uvwasi_t* uvwasi,
                                 uvwasi_riflags_t ri_flags,
                                 uvwasi_size_t* ro_datalen,
                                 uvwasi_roflags_t* ro_flags) {
-  /* TODO(cjihrig): Waiting to implement, pending
-                    https://github.com/WebAssembly/WASI/issues/4 */
-  UVWASI_DEBUG("uvwasi_sock_recv(uvwasi=%p, unimplemented)\n", uvwasi);
-  return UVWASI_ENOTSUP;
+  struct uvwasi_fd_wrap_t* wrap;
+  uvwasi_errno_t err = 0;
+  recv_data_t recv_data;
+
+  UVWASI_DEBUG("uvwasi_sock_recv(uvwasi=%p, sock=%d, ri_data=%p, "
+	       "ri_data_len=%d, ri_flags=%d, ro_datalen=%p, ro_flags=%p)\n",
+               uvwasi,
+               sock,
+               ri_data,
+               ri_data_len,
+               ri_flags,
+               ro_datalen,
+               ro_flags);
+
+  if (uvwasi == NULL || ri_data == NULL || ro_datalen == NULL || ro_flags == NULL)
+    return UVWASI_EINVAL;
+
+  if (ri_flags != 0)
+    return UVWASI_ENOTSUP;
+
+  err = uvwasi_fd_table_get(uvwasi->fds,
+                            sock,
+                            &wrap,
+                            UVWASI__RIGHTS_SOCKET_BASE,
+                            0);
+  if (err != UVWASI_ESUCCESS)
+    return err;
+
+  recv_data.base = ri_data->buf;
+  recv_data.len = ri_data->buf_len;
+  err = read_stream_sync(uvwasi, (uv_stream_t*) wrap->sock, &recv_data);
+  uv_mutex_unlock(&wrap->mutex);
+  if (err != 0) {
+    return err;
+  }
+
+  if (recv_data.nread == 0) {
+    return UVWASI_EAGAIN;
+  } else if (recv_data.nread < 0) {
+    return uvwasi__translate_uv_error(recv_data.nread);
+  }
+
+  *ro_datalen = recv_data.nread;
+  return UVWASI_ESUCCESS;
 }
 
 
@@ -2541,20 +2648,194 @@ uvwasi_errno_t uvwasi_sock_send(uvwasi_t* uvwasi,
                                 uvwasi_size_t si_data_len,
                                 uvwasi_siflags_t si_flags,
                                 uvwasi_size_t* so_datalen) {
-  /* TODO(cjihrig): Waiting to implement, pending
-                    https://github.com/WebAssembly/WASI/issues/4 */
-  UVWASI_DEBUG("uvwasi_sock_send(uvwasi=%p, unimplemented)\n", uvwasi);
-  return UVWASI_ENOTSUP;
-}
 
+  struct uvwasi_fd_wrap_t* wrap;
+  uvwasi_errno_t err = 0;
+  uv_buf_t* bufs;
+  int r = 0;
+
+  UVWASI_DEBUG("uvwasi_sock_send(uvwasi=%p, sock=%d, si_data=%p, "
+	       "si_data_len=%d, si_flags=%d, so_datalen=%p)\n",
+               uvwasi,
+               sock,
+               si_data,
+               si_data_len,
+               si_flags,
+               so_datalen);
+
+  if (uvwasi == NULL || si_data == NULL || so_datalen == NULL ||
+      si_flags != 0)
+    return UVWASI_EINVAL;
+
+  err = uvwasi_fd_table_get(uvwasi->fds,
+                            sock,
+                            &wrap,
+                            UVWASI__RIGHTS_SOCKET_BASE,
+                            0);
+  if (err != UVWASI_ESUCCESS)
+    return err;
+
+  err = uvwasi__setup_ciovs(uvwasi, &bufs, si_data, si_data_len);
+  if (err != UVWASI_ESUCCESS) {
+    uv_mutex_unlock(&wrap->mutex);
+    return err;
+  }
+
+  r = uv_try_write((uv_stream_t*) wrap->sock, bufs, si_data_len);
+  uvwasi__free(uvwasi, bufs);
+  uv_mutex_unlock(&wrap->mutex);
+  if (r < 0)
+    return uvwasi__translate_uv_error(r);
+
+  *so_datalen = (uvwasi_size_t) r;
+  return UVWASI_ESUCCESS;
+}
 
 uvwasi_errno_t uvwasi_sock_shutdown(uvwasi_t* uvwasi,
                                     uvwasi_fd_t sock,
                                     uvwasi_sdflags_t how) {
-  /* TODO(cjihrig): Waiting to implement, pending
-                    https://github.com/WebAssembly/WASI/issues/4 */
-  UVWASI_DEBUG("uvwasi_sock_shutdown(uvwasi=%p, unimplemented)\n", uvwasi);
-  return UVWASI_ENOTSUP;
+  struct uvwasi_fd_wrap_t* wrap;
+  uvwasi_errno_t err = 0;
+  shutdown_data_t shutdown_data;
+
+  if (how & ~UVWASI_SHUT_WR)
+    return UVWASI_ENOTSUP;
+
+  UVWASI_DEBUG("uvwasi_sock_shutdown(uvwasi=%p, sock=%d, how=%d)\n",
+               uvwasi,
+               sock,
+               how);
+
+  if (uvwasi == NULL)
+    return UVWASI_EINVAL;
+
+  err = uvwasi_fd_table_get(uvwasi->fds,
+                            sock,
+                            &wrap,
+                            UVWASI__RIGHTS_SOCKET_BASE,
+                            0);
+  if (err != UVWASI_ESUCCESS)
+    return err;
+
+  if (how & UVWASI_SHUT_WR) {
+    err = shutdown_stream_sync(uvwasi, (uv_stream_t*) wrap->sock, &shutdown_data);
+    if (err != UVWASI_ESUCCESS) {
+      uv_mutex_unlock(&wrap->mutex);
+      return err;
+    }
+  }
+
+  uv_mutex_unlock(&wrap->mutex);
+
+  if (shutdown_data.status != 0) 
+    return uvwasi__translate_uv_error(shutdown_data.status);
+
+  return UVWASI_ESUCCESS;
+}
+
+uvwasi_errno_t uvwasi_sock_accept(uvwasi_t* uvwasi,
+                                  uvwasi_fd_t sock,
+                                  uvwasi_fdflags_t flags,
+                                  uvwasi_fd_t* connect_sock) {
+  struct uvwasi_fd_wrap_t* wrap;
+  struct uvwasi_fd_wrap_t* connected_wrap;
+  uvwasi_errno_t err = 0;
+  uv_loop_t* sock_loop = NULL;
+  int r = 0;
+
+  UVWASI_DEBUG("uvwasi_sock_accept(uvwasi=%p, sock=%d, flags=%d, "
+               "connect_sock=%p)\n",
+               uvwasi,
+               sock,
+               flags,
+               connect_sock);
+
+  if (uvwasi == NULL || connect_sock == NULL)
+    return UVWASI_EINVAL;
+
+  if (flags & ~UVWASI_FDFLAG_NONBLOCK)
+    return UVWASI_ENOTSUP;
+
+  err = uvwasi_fd_table_get(uvwasi->fds,
+                            sock,
+                            &wrap,
+                            UVWASI__RIGHTS_SOCKET_BASE,
+                            0);
+  if (err != UVWASI_ESUCCESS)
+    return err;
+
+  sock_loop = uv_handle_get_loop((uv_handle_t*) wrap->sock);
+  uv_tcp_t* uv_connect_sock = (uv_tcp_t*) uvwasi__malloc(uvwasi, sizeof(uv_tcp_t));
+  uv_tcp_init(sock_loop, uv_connect_sock);
+
+  r = uv_accept((uv_stream_t*) wrap->sock, (uv_stream_t*) uv_connect_sock);
+  if (r != 0) {
+    if (r == UV_EAGAIN) {
+      // if not blocking then just return as we have to wait for a connection
+      if (flags & UVWASI_FDFLAG_NONBLOCK) {
+        err = free_handle_sync(uvwasi, (uv_handle_t*) uv_connect_sock);
+        uv_mutex_unlock(&wrap->mutex);
+        if (err != UVWASI_ESUCCESS) {
+          return err;
+	}
+        return UVWASI_EAGAIN;
+      }
+    } else {
+      err = uvwasi__translate_uv_error(r);
+      goto close_sock_and_error_exit;
+    }
+
+    // request was blocking and we have no connection yet. run
+    // the loop until a connection comes in
+    while (1) {
+      err = 0;
+      if (uv_run(sock_loop, UV_RUN_ONCE) == 0) {
+        err = UVWASI_ECONNABORTED;
+        goto close_sock_and_error_exit;
+      }
+
+      int r = uv_accept((uv_stream_t*) wrap->sock, (uv_stream_t*) uv_connect_sock);
+      if (r == UV_EAGAIN) {
+	// still no connection or error so run the loop again
+        continue;
+      }
+
+      if (r != 0) {
+	// An error occurred accepting the connection. Break out of the loop and
+	// report an error.
+        err = uvwasi__translate_uv_error(r);
+        goto close_sock_and_error_exit;
+      }
+
+      // if we get here a new connection was successfully accepted
+      break;
+    }
+  }
+
+  err = uvwasi_fd_table_insert(uvwasi,
+                               uvwasi->fds,
+                               -1,
+                               uv_connect_sock,
+                               NULL,
+                               NULL,
+                               UVWASI_FILETYPE_SOCKET_STREAM,
+                               UVWASI__RIGHTS_SOCKET_BASE,
+                               UVWASI__RIGHTS_SOCKET_INHERITING,
+                               1,
+                               &connected_wrap);
+
+  if (err != UVWASI_ESUCCESS)
+    goto close_sock_and_error_exit;
+
+  *connect_sock = connected_wrap->id;
+  uv_mutex_unlock(&wrap->mutex);
+  uv_mutex_unlock(&connected_wrap->mutex);
+  return UVWASI_ESUCCESS;
+
+close_sock_and_error_exit:
+  uvwasi__free(uvwasi, uv_connect_sock);
+  uv_mutex_unlock(&wrap->mutex);
+  return err;
 }
 
 
