@@ -14,15 +14,16 @@ class TimeoutProc(base.TestProcObserver):
     self._start = time.time()
 
   def _on_next_test(self, test):
-    self._on_event()
+    self.__on_event()
 
   def _on_result_for(self, test, result):
-    self._on_event()
+    self.__on_event()
 
   def _on_heartbeat(self):
-    self._on_event()
+    self.__on_event()
 
-  def _on_event(self):
+  def __on_event(self):
     if not self.is_stopped:
       if time.time() - self._start > self._duration_sec:
+        print('>>> Total timeout reached.')
         self.stop()
