@@ -24,6 +24,7 @@
 const {
   ArrayIsArray,
   Error,
+  MathMin,
   ObjectKeys,
   ObjectSetPrototypeOf,
   RegExpPrototypeExec,
@@ -50,7 +51,7 @@ const { ConnectionsList } = internalBinding('http_parser');
 const {
   kUniqueHeaders,
   parseUniqueHeadersOption,
-  OutgoingMessage
+  OutgoingMessage,
 } = require('_http_outgoing');
 const {
   kOutHeaders,
@@ -62,12 +63,12 @@ const {
 } = require('internal/http');
 const {
   defaultTriggerAsyncIdScope,
-  getOrSetAsyncId
+  getOrSetAsyncId,
 } = require('internal/async_hooks');
 const { IncomingMessage } = require('_http_incoming');
 const {
   connResetException,
-  codes
+  codes,
 } = require('internal/errors');
 const {
   ERR_HTTP_REQUEST_TIMEOUT,
@@ -75,19 +76,20 @@ const {
   ERR_HTTP_INVALID_STATUS_CODE,
   ERR_HTTP_SOCKET_ENCODING,
   ERR_INVALID_ARG_TYPE,
+  ERR_HTTP_SOCKET_ASSIGNED,
   ERR_INVALID_ARG_VALUE,
-  ERR_INVALID_CHAR
+  ERR_INVALID_CHAR,
 } = codes;
 const {
   validateInteger,
   validateBoolean,
   validateLinkHeaderValue,
-  validateObject
+  validateObject,
 } = require('internal/validators');
 const Buffer = require('buffer').Buffer;
 const {
   DTRACE_HTTP_SERVER_REQUEST,
-  DTRACE_HTTP_SERVER_RESPONSE
+  DTRACE_HTTP_SERVER_RESPONSE,
 } = require('internal/dtrace');
 const { setInterval, clearInterval } = require('timers');
 let debug = require('internal/util/debuglog').debuglog('http', (fn) => {
@@ -170,7 +172,7 @@ const STATUS_CODES = {
   508: 'Loop Detected',              // RFC 5842 7.2
   509: 'Bandwidth Limit Exceeded',
   510: 'Not Extended',               // RFC 2774 7
-  511: 'Network Authentication Required' // RFC 6585 6
+  511: 'Network Authentication Required', // RFC 6585 6
 };
 
 const kOnExecute = HTTPParser.kOnExecute | 0;
@@ -189,8 +191,8 @@ class HTTPServerAsyncResource {
   }
 }
 
-function ServerResponse(req) {
-  OutgoingMessage.call(this);
+function ServerResponse(req, options) {
+  OutgoingMessage.call(this, options);
 
   if (req.method === 'HEAD') this._hasBody = false;
 
@@ -261,8 +263,8 @@ function onServerResponseClose() {
   // array. That is, in the example below, b still gets called even though
   // it's been removed by a:
   //
-  //   var EventEmitter = require('events');
-  //   var obj = new EventEmitter();
+  //   const EventEmitter = require('events');
+  //   const obj = new EventEmitter();
   //   obj.on('event', a);
   //   obj.on('event', b);
   //   function a() { obj.removeListener('event', b) }
@@ -278,7 +280,9 @@ function onServerResponseClose() {
 }
 
 ServerResponse.prototype.assignSocket = function assignSocket(socket) {
-  assert(!socket._httpMessage);
+  if (socket._httpMessage) {
+    throw new ERR_HTTP_SOCKET_ASSIGNED();
+  }
   socket._httpMessage = this;
   socket.on('close', onServerResponseClose);
   this.socket = socket;
@@ -351,7 +355,7 @@ function writeHead(statusCode, reason, obj) {
     // writeHead(statusCode[, headers])
     if (!this.statusMessage)
       this.statusMessage = STATUS_CODES[statusCode] || 'unknown';
-    obj = reason;
+    obj ??= reason;
   }
   this.statusCode = statusCode;
 
@@ -451,11 +455,11 @@ function storeHTTPOptions(options) {
     validateInteger(headersTimeout, 'headersTimeout', 0);
     this.headersTimeout = headersTimeout;
   } else {
-    this.headersTimeout = 60_000; // 60 seconds
+    this.headersTimeout = MathMin(60_000, this.requestTimeout); // Minimum between 60 seconds or requestTimeout
   }
 
-  if (this.requestTimeout > 0 && this.headersTimeout > 0 && this.headersTimeout >= this.requestTimeout) {
-    throw new codes.ERR_OUT_OF_RANGE('headersTimeout', '< requestTimeout', headersTimeout);
+  if (this.requestTimeout > 0 && this.headersTimeout > 0 && this.headersTimeout > this.requestTimeout) {
+    throw new codes.ERR_OUT_OF_RANGE('headersTimeout', '<= requestTimeout', headersTimeout);
   }
 
   const keepAliveTimeout = options.keepAliveTimeout;
@@ -473,16 +477,37 @@ function storeHTTPOptions(options) {
   } else {
     this.connectionsCheckingInterval = 30_000; // 30 seconds
   }
+
+  const joinDuplicateHeaders = options.joinDuplicateHeaders;
+  if (joinDuplicateHeaders !== undefined) {
+    validateBoolean(joinDuplicateHeaders, 'options.joinDuplicateHeaders');
+  }
+  this.joinDuplicateHeaders = joinDuplicateHeaders;
+
+  const rejectNonStandardBodyWrites = options.rejectNonStandardBodyWrites;
+  if (rejectNonStandardBodyWrites !== undefined) {
+    validateBoolean(rejectNonStandardBodyWrites, 'options.rejectNonStandardBodyWrites');
+    this.rejectNonStandardBodyWrites = rejectNonStandardBodyWrites;
+  } else {
+    this.rejectNonStandardBodyWrites = false;
+  }
 }
 
-function setupConnectionsTracking(server) {
+function setupConnectionsTracking() {
   // Start connection handling
-  server[kConnections] = new ConnectionsList();
+  if (!this[kConnections]) {
+    this[kConnections] = new ConnectionsList();
+  }
 
   // This checker is started without checking whether any headersTimeout or requestTimeout is non zero
   // otherwise it would not be started if such timeouts are modified after createServer.
-  server[kConnectionsCheckingInterval] =
-    setInterval(checkConnections.bind(server), server.connectionsCheckingInterval).unref();
+  this[kConnectionsCheckingInterval] =
+    setInterval(checkConnections.bind(this), this.connectionsCheckingInterval).unref();
+}
+
+function httpServerPreClose(server) {
+  server.closeIdleConnections();
+  clearInterval(server[kConnectionsCheckingInterval]);
 }
 
 function Server(options, requestListener) {
@@ -502,7 +527,8 @@ function Server(options, requestListener) {
     this,
     { allowHalfOpen: true, noDelay: options.noDelay,
       keepAlive: options.keepAlive,
-      keepAliveInitialDelay: options.keepAliveInitialDelay });
+      keepAliveInitialDelay: options.keepAliveInitialDelay,
+      highWaterMark: options.highWaterMark });
 
   if (requestListener) {
     this.on('request', requestListener);
@@ -514,22 +540,27 @@ function Server(options, requestListener) {
   this.httpAllowHalfOpen = false;
 
   this.on('connection', connectionListener);
+  this.on('listening', setupConnectionsTracking);
 
   this.timeout = 0;
   this.maxHeadersCount = null;
   this.maxRequestsPerSocket = 0;
-  setupConnectionsTracking(this);
+
   this[kUniqueHeaders] = parseUniqueHeadersOption(options.uniqueHeaders);
 }
 ObjectSetPrototypeOf(Server.prototype, net.Server.prototype);
 ObjectSetPrototypeOf(Server, net.Server);
 
 Server.prototype.close = function() {
-  clearInterval(this[kConnectionsCheckingInterval]);
+  httpServerPreClose(this);
   ReflectApply(net.Server.prototype.close, this, arguments);
 };
 
 Server.prototype.closeAllConnections = function() {
+  if (!this[kConnections]) {
+    return;
+  }
+
   const connections = this[kConnections].all();
 
   for (let i = 0, l = connections.length; i < l; i++) {
@@ -538,6 +569,10 @@ Server.prototype.closeAllConnections = function() {
 };
 
 Server.prototype.closeIdleConnections = function() {
+  if (!this[kConnections]) {
+    return;
+  }
+
   const connections = this[kConnections].idle();
 
   for (let i = 0, l = connections.length; i < l; i++) {
@@ -597,7 +632,7 @@ function checkConnections() {
 
 function connectionListener(socket) {
   defaultTriggerAsyncIdScope(
-    getOrSetAsyncId(socket), connectionListenerInternal, this, socket
+    getOrSetAsyncId(socket), connectionListenerInternal, this, socket,
   );
 }
 
@@ -651,7 +686,7 @@ function connectionListenerInternal(server, socket) {
     // sent to the client.
     outgoingData: 0,
     requestsCount: 0,
-    keepAliveTimeoutSet: false
+    keepAliveTimeoutSet: false,
   };
   state.onData = socketOnData.bind(undefined,
                                    server, socket, parser, state);
@@ -800,20 +835,44 @@ function onParserTimeout(server, socket) {
 const noop = () => {};
 const badRequestResponse = Buffer.from(
   `HTTP/1.1 400 ${STATUS_CODES[400]}\r\n` +
-  'Connection: close\r\n\r\n', 'ascii'
+  'Connection: close\r\n\r\n', 'ascii',
 );
 const requestTimeoutResponse = Buffer.from(
   `HTTP/1.1 408 ${STATUS_CODES[408]}\r\n` +
-  'Connection: close\r\n\r\n', 'ascii'
+  'Connection: close\r\n\r\n', 'ascii',
 );
 const requestHeaderFieldsTooLargeResponse = Buffer.from(
   `HTTP/1.1 431 ${STATUS_CODES[431]}\r\n` +
-  'Connection: close\r\n\r\n', 'ascii'
+  'Connection: close\r\n\r\n', 'ascii',
 );
+
+const requestChunkExtensionsTooLargeResponse = Buffer.from(
+  `HTTP/1.1 413 ${STATUS_CODES[413]}\r\n` +
+  'Connection: close\r\n\r\n', 'ascii',
+);
+
+function warnUnclosedSocket() {
+  if (warnUnclosedSocket.emitted) {
+    return;
+  }
+
+  warnUnclosedSocket.emitted = true;
+  process.emitWarning(
+    'An error event has already been emitted on the socket. ' +
+    'Please use the destroy method on the socket while handling ' +
+    "a 'clientError' event.",
+  );
+}
+
 function socketOnError(e) {
   // Ignore further errors
   this.removeListener('error', socketOnError);
-  this.on('error', noop);
+
+  if (this.listenerCount('error', noop) === 0) {
+    this.on('error', noop);
+  } else {
+    warnUnclosedSocket();
+  }
 
   if (!this.server.emit('clientError', e, this)) {
     // Caution must be taken to avoid corrupting the remote peer.
@@ -827,6 +886,9 @@ function socketOnError(e) {
         case 'HPE_HEADER_OVERFLOW':
           response = requestHeaderFieldsTooLargeResponse;
           break;
+        case 'HPE_CHUNK_EXTENSIONS_OVERFLOW':
+          response = requestChunkExtensionsTooLargeResponse;
+          break;
         case 'ERR_HTTP_REQUEST_TIMEOUT':
           response = requestTimeoutResponse;
           break;
@@ -910,7 +972,7 @@ function resOnFinish(req, res, socket, state, server) {
       request: req,
       response: res,
       socket,
-      server
+      server,
     });
   }
 
@@ -988,7 +1050,11 @@ function parserOnIncoming(server, socket, state, req, keepAlive) {
     }
   }
 
-  const res = new server[kServerResponse](req);
+  const res = new server[kServerResponse](req,
+                                          {
+                                            highWaterMark: socket.writableHighWaterMark,
+                                            rejectNonStandardBodyWrites: server.rejectNonStandardBodyWrites,
+                                          });
   res._keepAliveTimeout = server.keepAliveTimeout;
   res._maxRequestsPerSocket = server.maxRequestsPerSocket;
   res._onPendingData = updateOutgoingData.bind(undefined,
@@ -1003,7 +1069,7 @@ function parserOnIncoming(server, socket, state, req, keepAlive) {
       request: req,
       response: res,
       socket,
-      server
+      server,
     });
   }
 
@@ -1136,5 +1202,7 @@ module.exports = {
   setupConnectionsTracking,
   storeHTTPOptions,
   _connectionListener: connectionListener,
-  kServerResponse
+  kServerResponse,
+  httpServerPreClose,
+  kConnectionsCheckingInterval,
 };
