@@ -35,13 +35,13 @@ const { IncomingMessage } = require('_http_incoming');
 const {
   validateHeaderName,
   validateHeaderValue,
-  OutgoingMessage
+  OutgoingMessage,
 } = require('_http_outgoing');
 const {
   _connectionListener,
   STATUS_CODES,
   Server,
-  ServerResponse
+  ServerResponse,
 } = require('_http_server');
 let maxHeaderSize;
 
@@ -52,6 +52,9 @@ let maxHeaderSize;
  *   ServerResponse?: ServerResponse;
  *   insecureHTTPParser?: boolean;
  *   maxHeaderSize?: number;
+ *   joinDuplicateHeaders?: boolean;
+ *   highWaterMark?: number;
+ *   rejectNonStandardBodyWrites?: boolean;
  *   }} [opts]
  * @param {Function} [requestListener]
  * @returns {Server}
@@ -128,7 +131,7 @@ module.exports = {
   setMaxIdleHTTPParsers(max) {
     validateInteger(max, 'max', 1);
     parsers.max = max;
-  }
+  },
 };
 
 ObjectDefineProperty(module.exports, 'maxHeaderSize', {
@@ -142,7 +145,7 @@ ObjectDefineProperty(module.exports, 'maxHeaderSize', {
     }
 
     return maxHeaderSize;
-  }
+  },
 });
 
 ObjectDefineProperty(module.exports, 'globalAgent', {
@@ -154,5 +157,5 @@ ObjectDefineProperty(module.exports, 'globalAgent', {
   },
   set(value) {
     httpAgent.globalAgent = value;
-  }
+  },
 });
