@@ -1,11 +1,10 @@
 'use strict';
 
 const {
-  ArrayPrototypeUnshift,
-  Error,
-  FunctionPrototypeBind,
+  ArrayPrototypeSlice,
+  ErrorCaptureStackTrace,
+  ObjectPrototypeHasOwnProperty,
   ObjectDefineProperty,
-  ReflectApply,
   Symbol,
 } = primordials;
 
@@ -39,7 +38,7 @@ const { setCallbackTrampoline } = async_wrap;
 const {
   async_hook_fields,
   async_id_fields,
-  execution_async_resources
+  execution_async_resources,
 } = async_wrap;
 // Store the pair executionAsyncId and triggerAsyncId in a AliasedFloat64Array
 // in Environment::AsyncHooks::async_ids_stack_ which tracks the resource
@@ -52,8 +51,6 @@ const {
   executionAsyncResource: executionAsyncResource_,
   clearAsyncIdStack,
 } = async_wrap;
-// For performance reasons, only track Promises when a hook is enabled.
-const { enablePromiseHook, disablePromiseHook } = async_wrap;
 // Properties in active_hooks are used to keep track of the set of hooks being
 // executed in case another hook is enabled/disabled. The new set of hooks is
 // then restored once the active set of hooks is finished executing.
@@ -77,7 +74,7 @@ const active_hooks = {
   // Keep track of the field counts held in active_hooks.tmp_array. Because the
   // async_hook_fields can't be reassigned, store each uint32 in an array that
   // is written back to async_hook_fields when active_hooks.array is restored.
-  tmp_fields: null
+  tmp_fields: null,
 };
 
 const { registerDestroyHook } = async_wrap;
@@ -87,14 +84,19 @@ const { resource_symbol, owner_symbol } = internalBinding('symbols');
 // Each constant tracks how many callbacks there are for any given step of
 // async execution. These are tracked so if the user didn't include callbacks
 // for a given step, that step can bail out early.
-const { kInit, kBefore, kAfter, kDestroy, kTotals, kPromiseResolve,
-        kCheck, kExecutionAsyncId, kAsyncIdCounter, kTriggerAsyncId,
-        kDefaultTriggerAsyncId, kStackLength, kUsesExecutionAsyncResource
+const {
+  kInit, kBefore, kAfter, kDestroy, kTotals, kPromiseResolve,
+  kCheck, kExecutionAsyncId, kAsyncIdCounter, kTriggerAsyncId,
+  kDefaultTriggerAsyncId, kStackLength, kUsesExecutionAsyncResource,
 } = async_wrap.constants;
 
+const { async_id_symbol,
+        trigger_async_id_symbol } = internalBinding('symbols');
+
+// Lazy load of internal/util/inspect;
+let inspect;
+
 // Used in AsyncHook and AsyncResource.
-const async_id_symbol = Symbol('asyncId');
-const trigger_async_id_symbol = Symbol('triggerAsyncId');
 const init_symbol = Symbol('init');
 const before_symbol = Symbol('before');
 const after_symbol = Symbol('after');
@@ -120,10 +122,10 @@ function callbackTrampoline(asyncId, resource, cb, ...args) {
 
   let result;
   if (asyncId === 0 && typeof domain_cb === 'function') {
-    ArrayPrototypeUnshift(args, cb);
-    result = ReflectApply(domain_cb, this, args);
+    args.unshift(cb);
+    result = domain_cb.apply(this, args);
   } else {
-    result = ReflectApply(cb, this, args);
+    result = cb.apply(this, args);
   }
 
   if (asyncId !== 0 && hasHooks(kAfter))
@@ -133,8 +135,6 @@ function callbackTrampoline(asyncId, resource, cb, ...args) {
   return result;
 }
 
-setCallbackTrampoline(callbackTrampoline);
-
 const topLevelResource = {};
 
 function executionAsyncResource() {
@@ -150,14 +150,18 @@ function executionAsyncResource() {
   return lookupPublicResource(resource);
 }
 
+function inspectExceptionValue(e) {
+  inspect ??= require('internal/util/inspect').inspect;
+  return { message: inspect(e) };
+}
+
 // Used to fatally abort the process if a callback throws.
 function fatalError(e) {
-  if (typeof e.stack === 'string') {
+  if (typeof e?.stack === 'string') {
     process._rawDebug(e.stack);
   } else {
-    const o = { message: e };
-    // eslint-disable-next-line no-restricted-syntax
-    Error.captureStackTrace(o, fatalError);
+    const o = inspectExceptionValue(e);
+    ErrorCaptureStackTrace(o, fatalError);
     process._rawDebug(o.stack);
   }
 
@@ -190,11 +194,12 @@ function emitInitNative(asyncId, type, triggerAsyncId, resource) {
   try {
     // Using var here instead of let because "for (var ...)" is faster than let.
     // Refs: https://github.com/nodejs/node/pull/30380#issuecomment-552948364
+    // eslint-disable-next-line no-var
     for (var i = 0; i < active_hooks.array.length; i++) {
       if (typeof active_hooks.array[i][init_symbol] === 'function') {
         active_hooks.array[i][init_symbol](
           asyncId, type, triggerAsyncId,
-          resource
+          resource,
         );
       }
     }
@@ -222,6 +227,7 @@ function emitHook(symbol, asyncId) {
   try {
     // Using var here instead of let because "for (var ...)" is faster than let.
     // Refs: https://github.com/nodejs/node/pull/30380#issuecomment-552948364
+    // eslint-disable-next-line no-var
     for (var i = 0; i < active_hooks.array.length; i++) {
       if (typeof active_hooks.array[i][symbol] === 'function') {
         active_hooks.array[i][symbol](asyncId);
@@ -243,11 +249,12 @@ function emitHook(symbol, asyncId) {
 }
 
 function emitHookFactory(symbol, name) {
-  const fn = FunctionPrototypeBind(emitHook, undefined, symbol);
+  const fn = emitHook.bind(undefined, symbol);
 
   // Set the name property of the function as it looks good in the stack trace.
   ObjectDefineProperty(fn, 'name', {
-    value: name
+    __proto__: null,
+    value: name,
   });
   return fn;
 }
@@ -268,7 +275,7 @@ function getHookArrays() {
 
 
 function storeActiveHooks() {
-  active_hooks.tmp_array = active_hooks.array.slice();
+  active_hooks.tmp_array = ArrayPrototypeSlice(active_hooks.array);
   // Don't want to make the assumption that kInit to kDestroy are indexes 0 to
   // 4. So do this the long way.
   active_hooks.tmp_fields = [];
@@ -294,27 +301,110 @@ function restoreActiveHooks() {
   active_hooks.tmp_fields = null;
 }
 
+function trackPromise(promise, parent) {
+  if (promise[async_id_symbol]) {
+    return;
+  }
+
+  // Get trigger id from parent async id before making the async id of the
+  // child so if a new one must be made it will be lower than the child.
+  const triggerAsyncId = parent ? getOrSetAsyncId(parent) :
+    getDefaultTriggerAsyncId();
+
+  promise[async_id_symbol] = newAsyncId();
+  promise[trigger_async_id_symbol] = triggerAsyncId;
+}
+
+function promiseInitHook(promise, parent) {
+  trackPromise(promise, parent);
+  const asyncId = promise[async_id_symbol];
+  const triggerAsyncId = promise[trigger_async_id_symbol];
+  emitInitScript(asyncId, 'PROMISE', triggerAsyncId, promise);
+}
+
+function promiseInitHookWithDestroyTracking(promise, parent) {
+  promiseInitHook(promise, parent);
+  destroyTracking(promise, parent);
+}
+
+function destroyTracking(promise, parent) {
+  trackPromise(promise, parent);
+  const asyncId = promise[async_id_symbol];
+  registerDestroyHook(promise, asyncId);
+}
+
+function promiseBeforeHook(promise) {
+  trackPromise(promise);
+  const asyncId = promise[async_id_symbol];
+  const triggerId = promise[trigger_async_id_symbol];
+  emitBeforeScript(asyncId, triggerId, promise);
+}
+
+function promiseAfterHook(promise) {
+  trackPromise(promise);
+  const asyncId = promise[async_id_symbol];
+  if (hasHooks(kAfter)) {
+    emitAfterNative(asyncId);
+  }
+  if (asyncId === executionAsyncId()) {
+    // This condition might not be true if async_hooks was enabled during
+    // the promise callback execution.
+    // Popping it off the stack can be skipped in that case, because it is
+    // known that it would correspond to exactly one call with
+    // PromiseHookType::kBefore that was not witnessed by the PromiseHook.
+    popAsyncContext(asyncId);
+  }
+}
+
+function promiseResolveHook(promise) {
+  trackPromise(promise);
+  const asyncId = promise[async_id_symbol];
+  emitPromiseResolveNative(asyncId);
+}
 
 let wantPromiseHook = false;
 function enableHooks() {
   async_hook_fields[kCheck] += 1;
 
+  setCallbackTrampoline(callbackTrampoline);
+}
+
+let stopPromiseHook;
+function updatePromiseHookMode() {
   wantPromiseHook = true;
-  enablePromiseHook();
+  let initHook;
+  if (initHooksExist()) {
+    initHook = destroyHooksExist() ? promiseInitHookWithDestroyTracking :
+      promiseInitHook;
+  } else if (destroyHooksExist()) {
+    initHook = destroyTracking;
+  }
+  if (stopPromiseHook) stopPromiseHook();
+  const promiseHooks = require('internal/promise_hooks');
+  stopPromiseHook = promiseHooks.createHook({
+    init: initHook,
+    before: promiseBeforeHook,
+    after: promiseAfterHook,
+    settled: promiseResolveHooksExist() ? promiseResolveHook : undefined,
+  });
 }
 
 function disableHooks() {
   async_hook_fields[kCheck] -= 1;
 
   wantPromiseHook = false;
+
+  setCallbackTrampoline();
+
   // Delay the call to `disablePromiseHook()` because we might currently be
   // between the `before` and `after` calls of a Promise.
   enqueueMicrotask(disablePromiseHookIfNecessary);
 }
 
 function disablePromiseHookIfNecessary() {
-  if (!wantPromiseHook)
-    disablePromiseHook();
+  if (!wantPromiseHook && stopPromiseHook) {
+    stopPromiseHook();
+  }
 }
 
 // Internal Embedder API //
@@ -327,7 +417,7 @@ function newAsyncId() {
 }
 
 function getOrSetAsyncId(object) {
-  if (object.hasOwnProperty(async_id_symbol)) {
+  if (ObjectPrototypeHasOwnProperty(object, async_id_symbol)) {
     return object[async_id_symbol];
   }
 
@@ -351,17 +441,25 @@ function clearDefaultTriggerAsyncId() {
   async_id_fields[kDefaultTriggerAsyncId] = -1;
 }
 
-
+/**
+ * Sets a default top level trigger ID to be used
+ * @template {Array<unknown>} T
+ * @template {unknown} R
+ * @param {number} triggerAsyncId
+ * @param { (...T: args) => R } block
+ * @param  {T} args
+ * @returns {R}
+ */
 function defaultTriggerAsyncIdScope(triggerAsyncId, block, ...args) {
   if (triggerAsyncId === undefined)
-    return block(...args);
+    return block.apply(null, args);
   // CHECK(NumberIsSafeInteger(triggerAsyncId))
   // CHECK(triggerAsyncId > 0)
   const oldDefaultTriggerAsyncId = async_id_fields[kDefaultTriggerAsyncId];
   async_id_fields[kDefaultTriggerAsyncId] = triggerAsyncId;
 
   try {
-    return block(...args);
+    return block.apply(null, args);
   } finally {
     async_id_fields[kDefaultTriggerAsyncId] = oldDefaultTriggerAsyncId;
   }
@@ -387,6 +485,10 @@ function destroyHooksExist() {
   return hasHooks(kDestroy);
 }
 
+function promiseResolveHooksExist() {
+  return hasHooks(kPromiseResolve);
+}
+
 
 function emitInitScript(asyncId, type, triggerAsyncId, resource) {
   // Short circuit all checks for the common case. Which is that no hooks have
@@ -481,13 +583,14 @@ module.exports = {
   symbols: {
     async_id_symbol, trigger_async_id_symbol,
     init_symbol, before_symbol, after_symbol, destroy_symbol,
-    promise_resolve_symbol, owner_symbol
+    promise_resolve_symbol, owner_symbol,
   },
   constants: {
-    kInit, kBefore, kAfter, kDestroy, kTotals, kPromiseResolve
+    kInit, kBefore, kAfter, kDestroy, kTotals, kPromiseResolve,
   },
   enableHooks,
   disableHooks,
+  updatePromiseHookMode,
   clearDefaultTriggerAsyncId,
   clearAsyncIdStack,
   hasAsyncIdStack,
@@ -505,6 +608,8 @@ module.exports = {
   emitBefore: emitBeforeScript,
   emitAfter: emitAfterScript,
   emitDestroy: emitDestroyScript,
+  pushAsyncContext,
+  popAsyncContext,
   registerDestroyHook,
   useDomainTrampoline,
   nativeHooks: {
@@ -512,6 +617,9 @@ module.exports = {
     before: emitBeforeNative,
     after: emitAfterNative,
     destroy: emitDestroyNative,
-    promise_resolve: emitPromiseResolveNative
-  }
+    promise_resolve: emitPromiseResolveNative,
+  },
+  asyncWrap: {
+    Providers: async_wrap.Providers,
+  },
 };
