@@ -37,10 +37,12 @@ const {
 } = errors.codes;
 
 const {
-  addSerializeCallback,
-  addDeserializeCallback,
-  isBuildingSnapshot,
-} = require('v8').startupSnapshot;
+  namespace: {
+    addSerializeCallback,
+    addDeserializeCallback,
+    isBuildingSnapshot,
+  },
+} = require('internal/v8/startup_snapshot');
 
 function validateTimeout(options) {
   const { timeout = -1 } = { ...options };
@@ -268,7 +270,7 @@ function emitInvalidHostnameWarning(hostname) {
       `The provided hostname "${hostname}" is not a valid ` +
       'hostname, and is supported in the dns module solely for compatibility.',
       'DeprecationWarning',
-      'DEP0118'
+      'DEP0118',
     );
     invalidHostnameWarningEmitted = true;
   }
@@ -283,6 +285,10 @@ function setDefaultResultOrder(value) {
   dnsOrder = value;
 }
 
+function getDefaultResultOrder() {
+  return dnsOrder;
+}
+
 function createResolverClass(resolver) {
   const resolveMap = ObjectCreate(null);
 
@@ -304,7 +310,7 @@ function createResolverClass(resolver) {
 
   return {
     resolveMap,
-    Resolver
+    Resolver,
   };
 }
 
@@ -345,8 +351,9 @@ module.exports = {
   validateTries,
   emitInvalidHostnameWarning,
   getDefaultVerbatim,
+  getDefaultResultOrder,
   setDefaultResultOrder,
   errorCodes,
   createResolverClass,
-  initializeDns
+  initializeDns,
 };
