@@ -8,7 +8,7 @@ const {
   DTRACE_HTTP_SERVER_REQUEST = () => {},
   DTRACE_HTTP_SERVER_RESPONSE = () => {},
   DTRACE_NET_SERVER_CONNECTION = () => {},
-  DTRACE_NET_STREAM_END = () => {}
+  DTRACE_NET_STREAM_END = () => {},
 } = (config.hasDtrace ? internalBinding('dtrace') : {});
 
 module.exports = {
@@ -17,5 +17,5 @@ module.exports = {
   DTRACE_HTTP_SERVER_REQUEST,
   DTRACE_HTTP_SERVER_RESPONSE,
   DTRACE_NET_SERVER_CONNECTION,
-  DTRACE_NET_STREAM_END
+  DTRACE_NET_STREAM_END,
 };
