@@ -10,7 +10,6 @@ const {
   FunctionPrototypeCall,
   Number,
   NumberIsFinite,
-  NumberIsInteger,
   MathMin,
   MathRound,
   ObjectIs,
@@ -22,6 +21,7 @@ const {
   StringPrototypeEndsWith,
   StringPrototypeIncludes,
   Symbol,
+  TypedArrayPrototypeAt,
   TypedArrayPrototypeIncludes,
 } = primordials;
 
@@ -33,10 +33,10 @@ const {
     ERR_INCOMPATIBLE_OPTION_PAIR,
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_ARG_VALUE,
-    ERR_OUT_OF_RANGE
+    ERR_OUT_OF_RANGE,
   },
   hideStackFrames,
-  uvException
+  uvException,
 } = require('internal/errors');
 const {
   isArrayBufferView,
@@ -97,13 +97,13 @@ const {
     UV_DIRENT_FIFO,
     UV_DIRENT_SOCKET,
     UV_DIRENT_CHAR,
-    UV_DIRENT_BLOCK
+    UV_DIRENT_BLOCK,
   },
   os: {
     errno: {
-      EISDIR
-    }
-  }
+      EISDIR,
+    },
+  },
 } = internalBinding('constants');
 
 // The access modes can be any of F_OK, R_OK, W_OK or X_OK. Some might not be
@@ -120,7 +120,7 @@ const kMinimumCopyMode = MathMin(
   kDefaultCopyMode,
   COPYFILE_EXCL,
   COPYFILE_FICLONE,
-  COPYFILE_FICLONE_FORCE
+  COPYFILE_FICLONE_FORCE,
 );
 const kMaximumCopyMode = COPYFILE_EXCL |
                          COPYFILE_FICLONE |
@@ -161,8 +161,9 @@ function assertEncoding(encoding) {
 }
 
 class Dirent {
-  constructor(name, type) {
+  constructor(name, type, path) {
     this.name = name;
+    this.path = path;
     this[kType] = type;
   }
 
@@ -196,8 +197,8 @@ class Dirent {
 }
 
 class DirentFromStats extends Dirent {
-  constructor(name, stats) {
-    super(name, null);
+  constructor(name, stats, path) {
+    super(name, null, path);
     this[kStats] = stats;
   }
 }
@@ -232,7 +233,7 @@ function join(path, name) {
   }
 
   if (typeof path === 'string' && typeof name === 'string') {
-    return pathModule.join(path, name);
+    return pathModule.basename(path) === name ? path : pathModule.join(path, name);
   }
 
   if (isUint8Array(path) && isUint8Array(name)) {
@@ -267,13 +268,13 @@ function getDirents(path, { 0: names, 1: types }, callback) {
             callback(err);
             return;
           }
-          names[idx] = new DirentFromStats(name, stats);
+          names[idx] = new DirentFromStats(name, stats, path);
           if (--toFinish === 0) {
             callback(null, names);
           }
         });
       } else {
-        names[i] = new Dirent(names[i], types[i]);
+        names[i] = new Dirent(names[i], types[i], path);
       }
     }
     if (toFinish === 0) {
@@ -303,16 +304,17 @@ function getDirent(path, name, type, callback) {
           callback(err);
           return;
         }
-        callback(null, new DirentFromStats(name, stats));
+        callback(null, new DirentFromStats(name, stats, filepath));
       });
     } else {
-      callback(null, new Dirent(name, type));
+      callback(null, new Dirent(name, type, path));
     }
   } else if (type === UV_DIRENT_UNKNOWN) {
-    const stats = lazyLoadFs().lstatSync(join(path, name));
-    return new DirentFromStats(name, stats);
+    const filepath = join(path, name);
+    const stats = lazyLoadFs().lstatSync(filepath);
+    return new DirentFromStats(name, stats, path);
   } else {
-    return new Dirent(name, type);
+    return new Dirent(name, type, path);
   }
 }
 
@@ -335,6 +337,7 @@ function getOptions(options, defaultOptions = kEmptyObject) {
   if (options.signal !== undefined) {
     validateAbortSignal(options.signal, 'options.signal');
   }
+
   return options;
 }
 
@@ -372,7 +375,7 @@ const nullCheck = hideStackFrames((path, propName, throwError = true) => {
   const err = new ERR_INVALID_ARG_VALUE(
     propName,
     path,
-    'must be a string or Uint8Array without null bytes'
+    'must be a string or Uint8Array without null bytes',
   );
   if (throwError) {
     throw err;
@@ -543,7 +546,7 @@ function getStatsFromBinding(stats, offset = 0) {
       nsFromTimeSpecBigInt(stats[10 + offset], stats[11 + offset]),
       nsFromTimeSpecBigInt(stats[12 + offset], stats[13 + offset]),
       nsFromTimeSpecBigInt(stats[14 + offset], stats[15 + offset]),
-      nsFromTimeSpecBigInt(stats[16 + offset], stats[17 + offset])
+      nsFromTimeSpecBigInt(stats[16 + offset], stats[17 + offset]),
     );
   }
   return new Stats(
@@ -554,7 +557,25 @@ function getStatsFromBinding(stats, offset = 0) {
     msFromTimeSpec(stats[10 + offset], stats[11 + offset]),
     msFromTimeSpec(stats[12 + offset], stats[13 + offset]),
     msFromTimeSpec(stats[14 + offset], stats[15 + offset]),
-    msFromTimeSpec(stats[16 + offset], stats[17 + offset])
+    msFromTimeSpec(stats[16 + offset], stats[17 + offset]),
+  );
+}
+
+class StatFs {
+  constructor(type, bsize, blocks, bfree, bavail, files, ffree) {
+    this.type = type;
+    this.bsize = bsize;
+    this.blocks = blocks;
+    this.bfree = bfree;
+    this.bavail = bavail;
+    this.files = files;
+    this.ffree = ffree;
+  }
+}
+
+function getStatFsFromBinding(stats) {
+  return new StatFs(
+    stats[0], stats[1], stats[2], stats[3], stats[4], stats[5], stats[6],
   );
 }
 
@@ -650,7 +671,7 @@ const validateOffsetLengthRead = hideStackFrames(
       throw new ERR_OUT_OF_RANGE('length',
                                  `<= ${bufferLength - offset}`, length);
     }
-  }
+  },
 );
 
 const validateOffsetLengthWrite = hideStackFrames(
@@ -668,7 +689,7 @@ const validateOffsetLengthWrite = hideStackFrames(
     }
 
     validateInt32(length, 'length', 0);
-  }
+  },
 );
 
 const validatePath = hideStackFrames((path, propName = 'path') => {
@@ -716,7 +737,9 @@ let nonPortableTemplateWarn = true;
 function warnOnNonPortableTemplate(template) {
   // Template strings passed to the mkdtemp() family of functions should not
   // end with 'X' because they are handled inconsistently across platforms.
-  if (nonPortableTemplateWarn && StringPrototypeEndsWith(template, 'X')) {
+  if (nonPortableTemplateWarn &&
+    ((typeof template === 'string' && StringPrototypeEndsWith(template, 'X')) ||
+    (typeof template !== 'string' && TypedArrayPrototypeAt(template, -1) === 0x58))) {
     process.emitWarning('mkdtemp() templates ending with X are not portable. ' +
                         'For details see: https://nodejs.org/api/fs.html');
     nonPortableTemplateWarn = false;
@@ -737,7 +760,7 @@ const defaultRmOptions = {
   recursive: false,
   force: false,
   retryDelay: 100,
-  maxRetries: 0
+  maxRetries: 0,
 };
 
 const defaultRmdirOptions = {
@@ -757,6 +780,7 @@ const validateCpOptions = hideStackFrames((options) => {
   validateBoolean(options.preserveTimestamps, 'options.preserveTimestamps');
   validateBoolean(options.recursive, 'options.recursive');
   validateBoolean(options.verbatimSymlinks, 'options.verbatimSymlinks');
+  options.mode = getValidMode(options.mode, 'copyFile');
   if (options.dereference === true && options.verbatimSymlinks === true) {
     throw new ERR_INCOMPATIBLE_OPTION_PAIR('dereference', 'verbatimSymlinks');
   }
@@ -788,7 +812,7 @@ const validateRmOptions = hideStackFrames((path, options, expectDir, cb) => {
         message: 'is a directory',
         path,
         syscall: 'rm',
-        errno: EISDIR
+        errno: EISDIR,
       }));
     }
     return cb(null, options);
@@ -813,7 +837,7 @@ const validateRmOptionsSync = hideStackFrames((path, options, expectDir) => {
         message: 'is a directory',
         path,
         syscall: 'rm',
-        errno: EISDIR
+        errno: EISDIR,
       });
     }
   }
@@ -828,7 +852,7 @@ function emitRecursiveRmdirWarning() {
       'In future versions of Node.js, fs.rmdir(path, { recursive: true }) ' +
       'will be removed. Use fs.rm(path, { recursive: true }) instead',
       'DeprecationWarning',
-      'DEP0147'
+      'DEP0147',
     );
     recursiveRmdirWarned = true;
   }
@@ -863,14 +887,8 @@ const getValidMode = hideStackFrames((mode, type) => {
   if (mode == null) {
     return def;
   }
-  if (NumberIsInteger(mode) && mode >= min && mode <= max) {
-    return mode;
-  }
-  if (typeof mode !== 'number') {
-    throw new ERR_INVALID_ARG_TYPE('mode', 'integer', mode);
-  }
-  throw new ERR_OUT_OF_RANGE(
-    'mode', `an integer >= ${min} && <= ${max}`, mode);
+  validateInteger(mode, 'mode', min, max);
+  return mode;
 });
 
 const validateStringAfterArrayBufferView = hideStackFrames((buffer, name) => {
@@ -890,7 +908,7 @@ const validateStringAfterArrayBufferView = hideStackFrames((buffer, name) => {
   throw new ERR_INVALID_ARG_TYPE(
     name,
     ['string', 'Buffer', 'TypedArray', 'DataView'],
-    buffer
+    buffer,
   );
 });
 
@@ -899,7 +917,7 @@ const validatePrimitiveStringAfterArrayBufferView = hideStackFrames((buffer, nam
     throw new ERR_INVALID_ARG_TYPE(
       name,
       ['string', 'Buffer', 'TypedArray', 'DataView'],
-      buffer
+      buffer,
     );
   }
 });
@@ -941,6 +959,7 @@ module.exports = {
   nullCheck,
   preprocessSymlinkDestination,
   realpathCacheKey: Symbol('realpathCacheKey'),
+  getStatFsFromBinding,
   getStatsFromBinding,
   stringToFlags,
   stringToSymlinkType,
@@ -957,5 +976,5 @@ module.exports = {
   validateRmdirOptions,
   validateStringAfterArrayBufferView,
   validatePrimitiveStringAfterArrayBufferView,
-  warnOnNonPortableTemplate
+  warnOnNonPortableTemplate,
 };
