@@ -1,51 +1,84 @@
 'use strict';
 
-const assert = require('assert');
-const Stream = require('stream');
-const Readable = Stream.Readable;
-const binding = process.binding('http2');
-const constants = binding.constants;
 const {
-  ERR_HTTP2_HEADERS_SENT,
-  ERR_HTTP2_INFO_STATUS_NOT_ALLOWED,
-  ERR_HTTP2_INVALID_HEADER_VALUE,
-  ERR_HTTP2_INVALID_STREAM,
-  ERR_HTTP2_NO_SOCKET_MANIPULATION,
-  ERR_HTTP2_PSEUDOHEADER_NOT_ALLOWED,
-  ERR_HTTP2_STATUS_INVALID,
-  ERR_INVALID_ARG_VALUE,
-  ERR_INVALID_CALLBACK,
-  ERR_INVALID_HTTP_TOKEN
-} = require('internal/errors').codes;
-const { validateString } = require('internal/validators');
-const { kSocket } = require('internal/http2/util');
+  ArrayIsArray,
+  ArrayPrototypePush,
+  Boolean,
+  FunctionPrototypeBind,
+  ObjectAssign,
+  ObjectCreate,
+  ObjectKeys,
+  ObjectPrototypeHasOwnProperty,
+  Proxy,
+  ReflectApply,
+  ReflectGetPrototypeOf,
+  StringPrototypeIncludes,
+  SafeArrayIterator,
+  StringPrototypeToLowerCase,
+  StringPrototypeTrim,
+  Symbol,
+} = primordials;
+
+const assert = require('internal/assert');
+const Stream = require('stream');
+const { Readable } = Stream;
+const {
+  constants: {
+    HTTP2_HEADER_AUTHORITY,
+    HTTP2_HEADER_CONNECTION,
+    HTTP2_HEADER_METHOD,
+    HTTP2_HEADER_PATH,
+    HTTP2_HEADER_SCHEME,
+    HTTP2_HEADER_STATUS,
+
+    HTTP_STATUS_CONTINUE,
+    HTTP_STATUS_EARLY_HINTS,
+    HTTP_STATUS_EXPECTATION_FAILED,
+    HTTP_STATUS_METHOD_NOT_ALLOWED,
+    HTTP_STATUS_OK,
+  },
+} = internalBinding('http2');
+const {
+  codes: {
+    ERR_HTTP2_HEADERS_SENT,
+    ERR_HTTP2_INFO_STATUS_NOT_ALLOWED,
+    ERR_HTTP2_INVALID_HEADER_VALUE,
+    ERR_HTTP2_INVALID_STREAM,
+    ERR_HTTP2_NO_SOCKET_MANIPULATION,
+    ERR_HTTP2_PSEUDOHEADER_NOT_ALLOWED,
+    ERR_HTTP2_STATUS_INVALID,
+    ERR_INVALID_ARG_VALUE,
+    ERR_INVALID_HTTP_TOKEN,
+    ERR_STREAM_WRITE_AFTER_END,
+  },
+  hideStackFrames,
+} = require('internal/errors');
+const {
+  validateFunction,
+  validateString,
+  validateLinkHeaderValue,
+  validateObject,
+} = require('internal/validators');
+const {
+  kSocket,
+  kRequest,
+  kProxySocket,
+  assertValidPseudoHeader,
+  getAuthority,
+} = require('internal/http2/util');
+const { _checkIsHttpToken: checkIsHttpToken } = require('_http_common');
 
 const kBeginSend = Symbol('begin-send');
 const kState = Symbol('state');
 const kStream = Symbol('stream');
-const kRequest = Symbol('request');
 const kResponse = Symbol('response');
 const kHeaders = Symbol('headers');
 const kRawHeaders = Symbol('rawHeaders');
 const kTrailers = Symbol('trailers');
 const kRawTrailers = Symbol('rawTrailers');
-const kProxySocket = Symbol('proxySocket');
 const kSetHeader = Symbol('setHeader');
 const kAborted = Symbol('aborted');
 
-const {
-  HTTP2_HEADER_AUTHORITY,
-  HTTP2_HEADER_METHOD,
-  HTTP2_HEADER_PATH,
-  HTTP2_HEADER_SCHEME,
-  HTTP2_HEADER_STATUS,
-
-  HTTP_STATUS_CONTINUE,
-  HTTP_STATUS_EXPECTATION_FAILED,
-  HTTP_STATUS_METHOD_NOT_ALLOWED,
-  HTTP_STATUS_OK
-} = constants;
-
 let statusMessageWarned = false;
 let statusConnectionHeaderWarned = false;
 
@@ -53,22 +86,22 @@ let statusConnectionHeaderWarned = false;
 // HTTP/2 implementation, intended to provide an interface that is as
 // close as possible to the current require('http') API
 
-function assertValidHeader(name, value) {
-  let err;
-  if (name === '' || typeof name !== 'string') {
-    err = new ERR_INVALID_HTTP_TOKEN('Header name', name);
-  } else if (isPseudoHeader(name)) {
-    err = new ERR_HTTP2_PSEUDOHEADER_NOT_ALLOWED();
-  } else if (value === undefined || value === null) {
-    err = new ERR_HTTP2_INVALID_HEADER_VALUE(value, name);
-  } else if (!isConnectionHeaderAllowed(name, value)) {
+const assertValidHeader = hideStackFrames((name, value) => {
+  if (name === '' ||
+      typeof name !== 'string' ||
+      StringPrototypeIncludes(name, ' ')) {
+    throw new ERR_INVALID_HTTP_TOKEN('Header name', name);
+  }
+  if (isPseudoHeader(name)) {
+    throw new ERR_HTTP2_PSEUDOHEADER_NOT_ALLOWED();
+  }
+  if (value === undefined || value === null) {
+    throw new ERR_HTTP2_INVALID_HEADER_VALUE(value, name);
+  }
+  if (!isConnectionHeaderAllowed(name, value)) {
     connectionHeaderMessageWarn();
   }
-  if (err !== undefined) {
-    Error.captureStackTrace(err, assertValidHeader);
-    throw err;
-  }
-}
+});
 
 function isPseudoHeader(name) {
   switch (name) {
@@ -87,14 +120,14 @@ function statusMessageWarn() {
   if (statusMessageWarned === false) {
     process.emitWarning(
       'Status message is not supported by HTTP/2 (RFC7540 8.1.2.4)',
-      'UnsupportedWarning'
+      'UnsupportedWarning',
     );
     statusMessageWarned = true;
   }
 }
 
 function isConnectionHeaderAllowed(name, value) {
-  return name !== constants.HTTP2_HEADER_CONNECTION ||
+  return name !== HTTP2_HEADER_CONNECTION ||
          value === 'trailers';
 }
 
@@ -104,7 +137,7 @@ function connectionHeaderMessageWarn() {
       'The provided connection header is not valid, ' +
       'the value will be dropped from the header and ' +
       'will never be in use.',
-      'UnsupportedWarning'
+      'UnsupportedWarning',
     );
     statusConnectionHeaderWarned = true;
   }
@@ -119,8 +152,9 @@ function onStreamData(chunk) {
 function onStreamTrailers(trailers, flags, rawTrailers) {
   const request = this[kRequest];
   if (request !== undefined) {
-    Object.assign(request[kTrailers], trailers);
-    request[kRawTrailers].push(...rawTrailers);
+    ObjectAssign(request[kTrailers], trailers);
+    ArrayPrototypePush(request[kRawTrailers],
+                       ...new SafeArrayIterator(rawTrailers));
   }
 }
 
@@ -132,7 +166,7 @@ function onStreamEnd() {
 }
 
 function onStreamError(error) {
-  // this is purposefully left blank
+  // This is purposefully left blank
   //
   // errors in compatibility mode are
   // not forwarded to the request
@@ -170,6 +204,11 @@ function resumeStream(stream) {
 }
 
 const proxySocketHandler = {
+  has(stream, prop) {
+    const ref = stream.session !== undefined ? stream.session[kSocket] : stream;
+    return (prop in stream) || (prop in ref);
+  },
+
   get(stream, prop) {
     switch (prop) {
       case 'on':
@@ -177,36 +216,41 @@ const proxySocketHandler = {
       case 'end':
       case 'emit':
       case 'destroy':
-        return stream[prop].bind(stream);
+        return FunctionPrototypeBind(stream[prop], stream);
       case 'writable':
       case 'destroyed':
         return stream[prop];
-      case 'readable':
+      case 'readable': {
         if (stream.destroyed)
           return false;
         const request = stream[kRequest];
         return request ? request.readable : stream.readable;
-      case 'setTimeout':
+      }
+      case 'setTimeout': {
         const session = stream.session;
         if (session !== undefined)
-          return session.setTimeout.bind(session);
-        return stream.setTimeout.bind(stream);
+          return FunctionPrototypeBind(session.setTimeout, session);
+        return FunctionPrototypeBind(stream.setTimeout, stream);
+      }
       case 'write':
       case 'read':
       case 'pause':
       case 'resume':
         throw new ERR_HTTP2_NO_SOCKET_MANIPULATION();
-      default:
+      default: {
         const ref = stream.session !== undefined ?
           stream.session[kSocket] : stream;
         const value = ref[prop];
-        return typeof value === 'function' ? value.bind(ref) : value;
+        return typeof value === 'function' ?
+          FunctionPrototypeBind(value, ref) :
+          value;
+      }
     }
   },
   getPrototypeOf(stream) {
     if (stream.session !== undefined)
-      return Reflect.getPrototypeOf(stream.session[kSocket]);
-    return Reflect.getPrototypeOf(stream);
+      return ReflectGetPrototypeOf(stream.session[kSocket]);
+    return ReflectGetPrototypeOf(stream);
   },
   set(stream, prop, value) {
     switch (prop) {
@@ -220,25 +264,27 @@ const proxySocketHandler = {
       case 'destroy':
         stream[prop] = value;
         return true;
-      case 'setTimeout':
+      case 'setTimeout': {
         const session = stream.session;
         if (session !== undefined)
           session.setTimeout = value;
         else
           stream.setTimeout = value;
         return true;
+      }
       case 'write':
       case 'read':
       case 'pause':
       case 'resume':
         throw new ERR_HTTP2_NO_SOCKET_MANIPULATION();
-      default:
+      default: {
         const ref = stream.session !== undefined ?
           stream.session[kSocket] : stream;
         ref[prop] = value;
         return true;
+      }
     }
-  }
+  },
 };
 
 function onStreamCloseRequest() {
@@ -251,7 +297,7 @@ function onStreamCloseRequest() {
   state.closed = true;
 
   req.push(null);
-  // if the user didn't interact with incoming data and didn't pipe it,
+  // If the user didn't interact with incoming data and didn't pipe it,
   // dump it for compatibility with http1
   if (!state.didRead && !req._readableState.resumeScheduled)
     req.resume();
@@ -271,11 +317,14 @@ function onStreamTimeout(kind) {
 
 class Http2ServerRequest extends Readable {
   constructor(stream, headers, options, rawHeaders) {
-    super(options);
+    super({ autoDestroy: false, ...options });
     this[kState] = {
       closed: false,
       didRead: false,
     };
+    // Headers in HTTP/1 are not initialized using Object.create(null) which,
+    // although preferable, would simply break too much code. Ergo header
+    // initialization using Object.create(null) in HTTP/2 is intentional.
     this[kHeaders] = headers;
     this[kRawHeaders] = rawHeaders;
     this[kTrailers] = {};
@@ -301,7 +350,8 @@ class Http2ServerRequest extends Readable {
   }
 
   get complete() {
-    return this._readableState.ended ||
+    return this[kAborted] ||
+           this.readableEnded ||
            this[kState].closed ||
            this[kStream].destroyed;
   }
@@ -367,14 +417,14 @@ class Http2ServerRequest extends Readable {
 
   set method(method) {
     validateString(method, 'method');
-    if (method.trim() === '')
+    if (StringPrototypeTrim(method) === '')
       throw new ERR_INVALID_ARG_VALUE('method', method);
 
     this[kHeaders][HTTP2_HEADER_METHOD] = method;
   }
 
   get authority() {
-    return this[kHeaders][HTTP2_HEADER_AUTHORITY];
+    return getAuthority(this[kHeaders]);
   }
 
   get scheme() {
@@ -390,9 +440,9 @@ class Http2ServerRequest extends Readable {
   }
 
   setTimeout(msecs, callback) {
-    if (this[kState].closed)
-      return;
-    this[kStream].setTimeout(msecs, callback);
+    if (!this[kState].closed)
+      this[kStream].setTimeout(msecs, callback);
+    return this;
   }
 }
 
@@ -428,16 +478,18 @@ class Http2ServerResponse extends Stream {
     this[kState] = {
       closed: false,
       ending: false,
+      destroyed: false,
       headRequest: false,
       sendDate: true,
       statusCode: HTTP_STATUS_OK,
     };
-    this[kHeaders] = Object.create(null);
-    this[kTrailers] = Object.create(null);
+    this[kHeaders] = ObjectCreate(null);
+    this[kTrailers] = ObjectCreate(null);
     this[kStream] = stream;
     stream[kProxySocket] = null;
     stream[kResponse] = this;
     this.writable = true;
+    this.req = stream[kRequest];
     stream.on('drain', onStreamDrain);
     stream.on('aborted', onStreamAbortedResponse);
     stream.on('close', onStreamCloseResponse);
@@ -452,18 +504,21 @@ class Http2ServerResponse extends Stream {
     return this.headersSent;
   }
 
+  get writableEnded() {
+    const state = this[kState];
+    return state.ending;
+  }
+
   get finished() {
-    const stream = this[kStream];
-    return stream.destroyed ||
-           stream._writableState.ended ||
-           this[kState].closed;
+    const state = this[kState];
+    return state.ending;
   }
 
   get socket() {
-    // this is compatible with http1 which removes socket reference
+    // This is compatible with http1 which removes socket reference
     // only from ServerResponse but not IncomingMessage
     if (this[kState].closed)
-      return;
+      return undefined;
 
     const stream = this[kStream];
     const proxySocket = stream[kProxySocket];
@@ -496,6 +551,22 @@ class Http2ServerResponse extends Stream {
     return this[kState].statusCode;
   }
 
+  get writableCorked() {
+    return this[kStream].writableCorked;
+  }
+
+  get writableHighWaterMark() {
+    return this[kStream].writableHighWaterMark;
+  }
+
+  get writableFinished() {
+    return this[kStream].writableFinished;
+  }
+
+  get writableLength() {
+    return this[kStream].writableLength;
+  }
+
   set statusCode(code) {
     code |= 0;
     if (code >= 100 && code < 200)
@@ -507,15 +578,15 @@ class Http2ServerResponse extends Stream {
 
   setTrailer(name, value) {
     validateString(name, 'name');
-    name = name.trim().toLowerCase();
+    name = StringPrototypeToLowerCase(StringPrototypeTrim(name));
     assertValidHeader(name, value);
     this[kTrailers][name] = value;
   }
 
   addTrailers(headers) {
-    const keys = Object.keys(headers);
+    const keys = ObjectKeys(headers);
     let key = '';
-    for (var i = 0; i < keys.length; i++) {
+    for (let i = 0; i < keys.length; i++) {
       key = keys[i];
       this.setTrailer(key, headers[key]);
     }
@@ -523,22 +594,23 @@ class Http2ServerResponse extends Stream {
 
   getHeader(name) {
     validateString(name, 'name');
-    name = name.trim().toLowerCase();
+    name = StringPrototypeToLowerCase(StringPrototypeTrim(name));
     return this[kHeaders][name];
   }
 
   getHeaderNames() {
-    return Object.keys(this[kHeaders]);
+    return ObjectKeys(this[kHeaders]);
   }
 
   getHeaders() {
-    return Object.assign({}, this[kHeaders]);
+    const headers = ObjectCreate(null);
+    return ObjectAssign(headers, this[kHeaders]);
   }
 
   hasHeader(name) {
     validateString(name, 'name');
-    name = name.trim().toLowerCase();
-    return Object.prototype.hasOwnProperty.call(this[kHeaders], name);
+    name = StringPrototypeToLowerCase(StringPrototypeTrim(name));
+    return ObjectPrototypeHasOwnProperty(this[kHeaders], name);
   }
 
   removeHeader(name) {
@@ -546,7 +618,14 @@ class Http2ServerResponse extends Stream {
     if (this[kStream].headersSent)
       throw new ERR_HTTP2_HEADERS_SENT();
 
-    name = name.trim().toLowerCase();
+    name = StringPrototypeToLowerCase(StringPrototypeTrim(name));
+
+    if (name === 'date') {
+      this[kState].sendDate = false;
+
+      return;
+    }
+
     delete this[kHeaders][name];
   }
 
@@ -559,13 +638,18 @@ class Http2ServerResponse extends Stream {
   }
 
   [kSetHeader](name, value) {
-    name = name.trim().toLowerCase();
+    name = StringPrototypeToLowerCase(StringPrototypeTrim(name));
     assertValidHeader(name, value);
 
     if (!isConnectionHeaderAllowed(name, value)) {
       return;
     }
 
+    if (name[0] === ':')
+      assertValidPseudoHeader(name);
+    else if (!checkIsHttpToken(name))
+      this.destroy(new ERR_INVALID_HTTP_TOKEN('Header name', name));
+
     this[kHeaders][name] = value;
   }
 
@@ -588,28 +672,35 @@ class Http2ServerResponse extends Stream {
   writeHead(statusCode, statusMessage, headers) {
     const state = this[kState];
 
-    if (state.closed)
-      throw new ERR_HTTP2_INVALID_STREAM();
+    if (state.closed || this.stream.destroyed)
+      return this;
     if (this[kStream].headersSent)
       throw new ERR_HTTP2_HEADERS_SENT();
 
-    if (this.stream.destroyed)
-      return this;
-
     if (typeof statusMessage === 'string')
       statusMessageWarn();
 
     if (headers === undefined && typeof statusMessage === 'object')
       headers = statusMessage;
 
-    var i;
-    if (Array.isArray(headers)) {
-      for (i = 0; i < headers.length; i++) {
-        const header = headers[i];
-        this[kSetHeader](header[0], header[1]);
+    let i;
+    if (ArrayIsArray(headers)) {
+      if (headers.length && ArrayIsArray(headers[0])) {
+        for (i = 0; i < headers.length; i++) {
+          const header = headers[i];
+          this[kSetHeader](header[0], header[1]);
+        }
+      } else {
+        if (headers.length % 2 !== 0) {
+          throw new ERR_INVALID_ARG_VALUE('headers', headers);
+        }
+
+        for (i = 0; i < headers.length; i += 2) {
+          this[kSetHeader](headers[i], headers[i + 1]);
+        }
       }
     } else if (typeof headers === 'object') {
-      const keys = Object.keys(headers);
+      const keys = ObjectKeys(headers);
       let key = '';
       for (i = 0; i < keys.length; i++) {
         key = keys[i];
@@ -623,24 +714,41 @@ class Http2ServerResponse extends Stream {
     return this;
   }
 
+  cork() {
+    this[kStream].cork();
+  }
+
+  uncork() {
+    this[kStream].uncork();
+  }
+
   write(chunk, encoding, cb) {
+    const state = this[kState];
+
     if (typeof encoding === 'function') {
       cb = encoding;
       encoding = 'utf8';
     }
 
-    if (this[kState].closed) {
-      const err = new ERR_HTTP2_INVALID_STREAM();
+    let err;
+    if (state.ending) {
+      err = new ERR_STREAM_WRITE_AFTER_END();
+    } else if (state.closed) {
+      err = new ERR_HTTP2_INVALID_STREAM();
+    } else if (state.destroyed) {
+      return false;
+    }
+
+    if (err) {
       if (typeof cb === 'function')
         process.nextTick(cb, err);
-      else
-        throw err;
-      return;
+      this.destroy(err);
+      return false;
     }
 
     const stream = this[kStream];
     if (!stream.headersSent)
-      this.writeHead(this[kState].statusCode);
+      this.writeHead(state.statusCode);
     return stream.write(chunk, encoding, cb);
   }
 
@@ -648,11 +756,6 @@ class Http2ServerResponse extends Stream {
     const stream = this[kStream];
     const state = this[kState];
 
-    if ((state.closed || state.ending) &&
-        state.headRequest === stream.headRequest) {
-      return this;
-    }
-
     if (typeof chunk === 'function') {
       cb = chunk;
       chunk = null;
@@ -661,15 +764,22 @@ class Http2ServerResponse extends Stream {
       encoding = 'utf8';
     }
 
+    if ((state.closed || state.ending) &&
+        state.headRequest === stream.headRequest) {
+      if (typeof cb === 'function') {
+        process.nextTick(cb);
+      }
+      return this;
+    }
+
     if (chunk !== null && chunk !== undefined)
       this.write(chunk, encoding);
 
-    const isFinished = this.finished;
     state.headRequest = stream.headRequest;
     state.ending = true;
 
     if (typeof cb === 'function') {
-      if (isFinished)
+      if (stream.writableEnded)
         this.once('finish', cb);
       else
         stream.once('finish', cb);
@@ -678,8 +788,8 @@ class Http2ServerResponse extends Stream {
     if (!stream.headersSent)
       this.writeHead(this[kState].statusCode);
 
-    if (isFinished)
-      onStreamCloseResponse.call(stream);
+    if (this[kState].closed || stream.destroyed)
+      ReflectApply(onStreamCloseResponse, stream, []);
     else
       stream.end();
 
@@ -687,8 +797,10 @@ class Http2ServerResponse extends Stream {
   }
 
   destroy(err) {
-    if (this[kState].closed)
+    if (this[kState].destroyed)
       return;
+
+    this[kState].destroyed = true;
     this[kStream].destroy(err);
   }
 
@@ -699,8 +811,7 @@ class Http2ServerResponse extends Stream {
   }
 
   createPushResponse(headers, callback) {
-    if (typeof callback !== 'function')
-      throw new ERR_INVALID_CALLBACK();
+    validateFunction(callback, 'callback');
     if (this[kState].closed) {
       process.nextTick(callback, new ERR_HTTP2_INVALID_STREAM());
       return;
@@ -721,6 +832,7 @@ class Http2ServerResponse extends Stream {
     const options = {
       endStream: state.ending,
       waitForTrailers: true,
+      sendDate: state.sendDate,
     };
     this[kStream].respond(headers, options);
   }
@@ -731,10 +843,41 @@ class Http2ServerResponse extends Stream {
     if (stream.headersSent || this[kState].closed)
       return false;
     stream.additionalHeaders({
-      [HTTP2_HEADER_STATUS]: HTTP_STATUS_CONTINUE
+      [HTTP2_HEADER_STATUS]: HTTP_STATUS_CONTINUE,
     });
     return true;
   }
+
+  writeEarlyHints(hints) {
+    validateObject(hints, 'hints');
+
+    const headers = ObjectCreate(null);
+
+    const linkHeaderValue = validateLinkHeaderValue(hints.link);
+
+    for (const key of ObjectKeys(hints)) {
+      if (key !== 'link') {
+        headers[key] = hints[key];
+      }
+    }
+
+    if (linkHeaderValue.length === 0) {
+      return false;
+    }
+
+    const stream = this[kStream];
+
+    if (stream.headersSent || this[kState].closed)
+      return false;
+
+    stream.additionalHeaders({
+      ...headers,
+      [HTTP2_HEADER_STATUS]: HTTP_STATUS_EARLY_HINTS,
+      'Link': linkHeaderValue,
+    });
+
+    return true;
+  }
 }
 
 function onServerStream(ServerRequest, ServerResponse,
