@@ -4,39 +4,43 @@ const {
   NumberParseInt,
   ObjectDefineProperties,
   ObjectDefineProperty,
-  ObjectGetOwnPropertyDescriptor,
   SafeMap,
-  SafeWeakMap,
   StringPrototypeStartsWith,
+  Symbol,
+  SymbolDispose,
+  SymbolAsyncDispose,
   globalThis,
 } = primordials;
 
 const {
   getOptionValue,
-  getEmbedderOptions,
   refreshOptions,
 } = require('internal/options');
 const { reconnectZeroFillToggle } = require('internal/buffer');
 const {
   defineOperation,
   exposeInterface,
+  exposeLazyInterfaces,
+  defineReplaceableLazyAttribute,
+  setupCoverageHooks,
 } = require('internal/util');
 
 const {
   ERR_MANIFEST_ASSERT_INTEGRITY,
 } = require('internal/errors').codes;
 const assert = require('internal/assert');
-
 const {
-  addSerializeCallback,
-  isBuildingSnapshot,
-} = require('v8').startupSnapshot;
+  namespace: {
+    addSerializeCallback,
+    isBuildingSnapshot,
+  },
+} = require('internal/v8/startup_snapshot');
 
 function prepareMainThreadExecution(expandArgv1 = false, initializeModules = true) {
-  prepareExecution({
+  return prepareExecution({
     expandArgv1,
     initializeModules,
-    isMainThread: true
+    isMainThread: true,
   });
 }
 
@@ -44,7 +48,7 @@ function prepareWorkerThreadExecution() {
   prepareExecution({
     expandArgv1: false,
     initializeModules: false,  // Will need to initialize it after policy setup
-    isMainThread: false
+    isMainThread: false,
   });
 }
 
@@ -54,8 +58,8 @@ function prepareExecution(options) {
   refreshRuntimeOptions();
   reconnectZeroFillToggle();
 
-  // Patch the process object with legacy properties and normalizations
-  patchProcessObject(expandArgv1);
+  // Patch the process object and get the resolved main entry point.
+  const mainEntry = patchProcessObject(expandArgv1);
   setupTraceCategoryState();
   setupPerfHooks();
   setupInspectorHooks();
@@ -63,23 +67,17 @@ function prepareExecution(options) {
   setupFetch();
   setupWebCrypto();
   setupCustomEvent();
-
-  // Resolve the coverage directory to an absolute path, and
-  // overwrite process.env so that the original path gets passed
-  // to child processes even when they switch cwd.
-  if (process.env.NODE_V8_COVERAGE) {
-    process.env.NODE_V8_COVERAGE =
-      setupCoverageHooks(process.env.NODE_V8_COVERAGE);
-  }
-
+  setupCodeCoverage();
   setupDebugEnv();
   // Process initial diagnostic reporting configuration, if present.
   initializeReport();
   initializeSourceMapsHandlers();
   initializeDeprecations();
-  initializeWASI();
+
   require('internal/dns/utils').initializeDns();
 
+  setupSymbolDisposePolyfill();
+
   if (isMainThread) {
     assert(internalBinding('worker').isMainThread);
     // Worker threads will get the manifest in the message handler.
@@ -115,13 +113,44 @@ function prepareExecution(options) {
   if (initializeModules) {
     setupUserModules();
   }
+
+  return mainEntry;
 }
 
-function setupUserModules() {
+function setupSymbolDisposePolyfill() {
+  // TODO(MoLow): Remove this polyfill once Symbol.dispose and Symbol.asyncDispose are available in V8.
+  // eslint-disable-next-line node-core/prefer-primordials
+  if (typeof Symbol.dispose !== 'symbol') {
+    ObjectDefineProperty(Symbol, 'dispose', {
+      __proto__: null,
+      configurable: false,
+      enumerable: false,
+      value: SymbolDispose,
+      writable: false,
+    });
+  }
+
+  // eslint-disable-next-line node-core/prefer-primordials
+  if (typeof Symbol.asyncDispose !== 'symbol') {
+    ObjectDefineProperty(Symbol, 'asyncDispose', {
+      __proto__: null,
+      configurable: false,
+      enumerable: false,
+      value: SymbolAsyncDispose,
+      writable: false,
+    });
+  }
+}
+
+function setupUserModules(isLoaderWorker = false) {
   initializeCJSLoader();
-  initializeESMLoader();
+  initializeESMLoader(isLoaderWorker);
   const CJSLoader = require('internal/modules/cjs/loader');
   assert(!CJSLoader.hasLoadedAnyUserCJSModule);
+  // Loader workers are responsible for doing this themselves.
+  if (isLoaderWorker) {
+    return;
+  }
   loadPreloadModules();
   // Need to be done after --require setup.
   initializeFrozenIntrinsics();
@@ -131,30 +160,43 @@ function refreshRuntimeOptions() {
   refreshOptions();
 }
 
+/**
+ * Patch the process object with legacy properties and normalizations.
+ * Replace `process.argv[0]` with `process.execPath`, preserving the original `argv[0]` value as `process.argv0`.
+ * Replace `process.argv[1]` with the resolved absolute file path of the entry point, if found.
+ * @param {boolean} expandArgv1 - Whether to replace `process.argv[1]` with the resolved absolute file path of
+ * the main entry point.
+ */
 function patchProcessObject(expandArgv1) {
   const binding = internalBinding('process_methods');
   binding.patchProcessObject(process);
 
   require('internal/process/per_thread').refreshHrtimeBuffer();
 
+  // Since we replace process.argv[0] below, preserve the original value in case the user needs it.
   ObjectDefineProperty(process, 'argv0', {
     __proto__: null,
     enumerable: true,
     // Only set it to true during snapshot building.
     configurable: getOptionValue('--build-snapshot'),
-    value: process.argv[0]
+    value: process.argv[0],
   });
 
   process.exitCode = undefined;
   process._exiting = false;
   process.argv[0] = process.execPath;
 
+  /** @type {string} */
+  let mainEntry;
+  // If requested, update process.argv[1] to replace whatever the user provided with the resolved absolute file path of
+  // the entry point.
   if (expandArgv1 && process.argv[1] &&
       !StringPrototypeStartsWith(process.argv[1], '-')) {
     // Expand process.argv[1] into a full path.
     const path = require('path');
     try {
-      process.argv[1] = path.resolve(process.argv[1]);
+      mainEntry = path.resolve(process.argv[1]);
+      process.argv[1] = mainEntry;
     } catch {
       // Continue regardless of error.
     }
@@ -181,6 +223,8 @@ function patchProcessObject(expandArgv1) {
   addReadOnlyProcessAlias('traceDeprecation', '--trace-deprecation');
   addReadOnlyProcessAlias('_breakFirstLine', '--inspect-brk', false);
   addReadOnlyProcessAlias('_breakNodeFirstLine', '--inspect-brk-node', false);
+
+  return mainEntry;
 }
 
 function addReadOnlyProcessAlias(name, option, enumerable = true) {
@@ -191,7 +235,7 @@ function addReadOnlyProcessAlias(name, option, enumerable = true) {
       writable: false,
       configurable: true,
       enumerable,
-      value
+      value,
     });
   }
 }
@@ -199,7 +243,7 @@ function addReadOnlyProcessAlias(name, option, enumerable = true) {
 function setupWarningHandler() {
   const {
     onWarning,
-    resetForSerialization
+    resetForSerialization,
   } = require('internal/process/warning');
   if (getOptionValue('--warnings') &&
     process.env.NODE_NO_WARNINGS !== '1') {
@@ -248,7 +292,7 @@ function setupFetch() {
       },
       set(value) {
         exposeInterface(globalThis, name, value);
-      }
+      },
     };
   }
 
@@ -260,8 +304,9 @@ function setupFetch() {
   });
 
   // The WebAssembly Web API: https://webassembly.github.io/spec/web-api
-  const { wasmStreamingCallback } = require('internal/wasm_web_api');
-  internalBinding('wasm_web_api').setImplementation(wasmStreamingCallback);
+  internalBinding('wasm_web_api').setImplementation((streamState, source) => {
+    require('internal/wasm_web_api').wasmStreamingCallback(streamState, source);
+  });
 }
 
 // TODO(aduh95): move this to internal/bootstrap/browser when the CLI flag is
@@ -272,19 +317,27 @@ function setupWebCrypto() {
     return;
   }
 
-  let webcrypto;
-  ObjectDefineProperty(globalThis, 'crypto',
-                       { __proto__: null, ...ObjectGetOwnPropertyDescriptor({
-                         get crypto() {
-                           webcrypto ??= require('internal/crypto/webcrypto');
-                           return webcrypto.crypto;
-                         }
-                       }, 'crypto') });
   if (internalBinding('config').hasOpenSSL) {
-    webcrypto ??= require('internal/crypto/webcrypto');
-    exposeInterface(globalThis, 'Crypto', webcrypto.Crypto);
-    exposeInterface(globalThis, 'CryptoKey', webcrypto.CryptoKey);
-    exposeInterface(globalThis, 'SubtleCrypto', webcrypto.SubtleCrypto);
+    defineReplaceableLazyAttribute(
+      globalThis, 'internal/crypto/webcrypto', ['crypto'], false,
+    );
+    exposeLazyInterfaces(
+      globalThis, 'internal/crypto/webcrypto',
+      ['Crypto', 'CryptoKey', 'SubtleCrypto'],
+    );
+  }
+}
+
+function setupCodeCoverage() {
+  // Resolve the coverage directory to an absolute path, and
+  // overwrite process.env so that the original path gets passed
+  // to child processes even when they switch cwd. Don't do anything if the
+  // --experimental-test-coverage flag is present, as the test runner will
+  // handle coverage.
+  if (process.env.NODE_V8_COVERAGE &&
+      !getOptionValue('--experimental-test-coverage')) {
+    process.env.NODE_V8_COVERAGE =
+      setupCoverageHooks(process.env.NODE_V8_COVERAGE);
   }
 }
 
@@ -299,27 +352,6 @@ function setupCustomEvent() {
   exposeInterface(globalThis, 'CustomEvent', CustomEvent);
 }
 
-// Setup User-facing NODE_V8_COVERAGE environment variable that writes
-// ScriptCoverage to a specified file.
-function setupCoverageHooks(dir) {
-  const cwd = require('internal/process/execution').tryGetCwd();
-  const { resolve } = require('path');
-  const coverageDirectory = resolve(cwd, dir);
-  const { sourceMapCacheToObject } =
-    require('internal/source_map/source_map_cache');
-
-  if (process.features.inspector) {
-    internalBinding('profiler').setCoverageDirectory(coverageDirectory);
-    internalBinding('profiler').setSourceMapCacheGetter(sourceMapCacheToObject);
-  } else {
-    process.emitWarning('The inspector is disabled, ' +
-                        'coverage could not be collected',
-                        'Warning');
-    return '';
-  }
-  return coverageDirectory;
-}
-
 function setupStacktracePrinterOnSigint() {
   if (!getOptionValue('--trace-sigint')) {
     return;
@@ -331,29 +363,30 @@ function setupStacktracePrinterOnSigint() {
 }
 
 function initializeReport() {
-  const { report } = require('internal/process/report');
   ObjectDefineProperty(process, 'report', {
     __proto__: null,
     enumerable: true,
     configurable: true,
     get() {
+      const { report } = require('internal/process/report');
       return report;
-    }
+    },
   });
 }
 
 function setupDebugEnv() {
   require('internal/util/debuglog').initializeDebugEnv(process.env.NODE_DEBUG);
   if (getOptionValue('--expose-internals')) {
-    require('internal/bootstrap/loaders').BuiltinModule.exposeInternals();
+    require('internal/bootstrap/realm').BuiltinModule.exposeInternals();
   }
 }
 
 // This has to be called after initializeReport() is called
 function initializeReportSignalHandlers() {
-  const { addSignalHandler } = require('internal/process/report');
-
-  addSignalHandler();
+  if (getOptionValue('--report-on-signal')) {
+    const { addSignalHandler } = require('internal/process/report');
+    addSignalHandler();
+  }
 }
 
 function initializeHeapSnapshotSignalHandlers() {
@@ -386,7 +419,6 @@ function setupTraceCategoryState() {
 }
 
 function setupPerfHooks() {
-  require('internal/perf/performance').refreshTimeOrigin();
   require('internal/perf/utils').refreshTimeOrigin();
 }
 
@@ -399,7 +431,7 @@ function setupInspectorHooks() {
   if (internalBinding('config').hasInspector) {
     const {
       enable,
-      disable
+      disable,
     } = require('internal/inspector_async_hook');
     internalBinding('inspector').registerAsyncHook(enable, disable);
   }
@@ -454,7 +486,7 @@ function initializeDeprecations() {
       writable: false,
       enumerable: true,
       configurable: true,
-      value: noBrowserGlobals
+      value: noBrowserGlobals,
     });
   }
 
@@ -507,7 +539,7 @@ function readPolicyFromDisk() {
     // no bare specifiers for now
     let manifestURL;
     if (require('path').isAbsolute(experimentalPolicy)) {
-      manifestURL = new URL(`file://${experimentalPolicy}`);
+      manifestURL = pathToFileURL(experimentalPolicy);
     } else {
       const cwdURL = pathToFileURL(process.cwd());
       cwdURL.pathname += '/';
@@ -525,7 +557,7 @@ function readPolicyFromDisk() {
       for (let i = 0; i < integrityEntries.length; i++) {
         const {
           algorithm,
-          value: expected
+          value: expected,
         } = integrityEntries[i];
         const hash = createHash(algorithm);
         hash.update(src);
@@ -542,43 +574,19 @@ function readPolicyFromDisk() {
       }
     }
     return {
-      manifestSrc: src, manifestURL: manifestURL.href
+      manifestSrc: src, manifestURL: manifestURL.href,
     };
   }
 }
 
-function initializeWASI() {
-  const { BuiltinModule } = require('internal/bootstrap/loaders');
-  const mod = BuiltinModule.map.get('wasi');
-  mod.canBeRequiredByUsers =
-    getOptionValue('--experimental-wasi-unstable-preview1');
-}
-
 function initializeCJSLoader() {
-  const CJSLoader = require('internal/modules/cjs/loader');
-  if (!getEmbedderOptions().noGlobalSearchPaths) {
-    CJSLoader.Module._initPaths();
-  }
-  // TODO(joyeecheung): deprecate this in favor of a proper hook?
-  CJSLoader.Module.runMain =
-    require('internal/modules/run_main').executeUserEntryPoint;
+  const { initializeCJS } = require('internal/modules/cjs/loader');
+  initializeCJS();
 }
 
-function initializeESMLoader() {
-  // Create this WeakMap in js-land because V8 has no C++ API for WeakMap.
-  internalBinding('module_wrap').callbackMap = new SafeWeakMap();
-
-  if (getEmbedderOptions().shouldNotRegisterESMLoader) return;
-
-  const {
-    setImportModuleDynamicallyCallback,
-    setInitializeImportMetaObjectCallback
-  } = internalBinding('module_wrap');
-  const esm = require('internal/process/esm_loader');
-  // Setup per-isolate callbacks that locate data or callbacks that we keep
-  // track of for different ESM modules.
-  setInitializeImportMetaObjectCallback(esm.initializeImportMetaObject);
-  setImportModuleDynamicallyCallback(esm.importModuleDynamicallyCallback);
+function initializeESMLoader(isLoaderWorker) {
+  const { initializeESM } = require('internal/modules/esm/utils');
+  initializeESM(isLoaderWorker);
 
   // Patch the vm module when --experimental-vm-modules is on.
   // Please update the comments in vm.js when this block changes.
@@ -594,11 +602,10 @@ function initializeESMLoader() {
 }
 
 function initializeSourceMapsHandlers() {
-  const { setSourceMapsEnabled, getSourceMapsEnabled } =
-    require('internal/source_map/source_map_cache');
-  process.setSourceMapsEnabled = setSourceMapsEnabled;
-  // Initialize the environment flag of source maps.
-  getSourceMapsEnabled();
+  const {
+    setSourceMapsEnabled,
+  } = require('internal/source_map/source_map_cache');
+  setSourceMapsEnabled(getOptionValue('--enable-source-maps'));
 }
 
 function initializeFrozenIntrinsics() {
@@ -615,7 +622,7 @@ function loadPreloadModules() {
   if (preloadModules && preloadModules.length > 0) {
     const {
       Module: {
-        _preloadModules
+        _preloadModules,
       },
     } = require('internal/modules/cjs/loader');
     _preloadModules(preloadModules);
@@ -630,5 +637,7 @@ module.exports = {
   setupUserModules,
   prepareMainThreadExecution,
   prepareWorkerThreadExecution,
-  markBootstrapComplete
+  markBootstrapComplete,
+  loadPreloadModules,
+  initializeFrozenIntrinsics,
 };
