@@ -1,9 +1,12 @@
 'use strict';
 
 const {
+  ArrayPrototypePush,
+  ArrayPrototypeShift,
   Error,
   ObjectDefineProperty,
-  WeakMap,
+  ObjectPrototypeHasOwnProperty,
+  SafeWeakMap,
 } = primordials;
 
 const {
@@ -12,40 +15,62 @@ const {
     kPromiseRejectWithNoHandler,
     kPromiseHandlerAddedAfterReject,
     kPromiseResolveAfterResolved,
-    kPromiseRejectAfterResolved
+    kPromiseRejectAfterResolved,
   },
-  setPromiseRejectCallback
+  setPromiseRejectCallback,
 } = internalBinding('task_queue');
 
+const { deprecate } = require('internal/util');
+
 const {
   noSideEffectsToString,
-  triggerUncaughtException
+  triggerUncaughtException,
 } = internalBinding('errors');
 
+const {
+  pushAsyncContext,
+  popAsyncContext,
+  symbols: {
+    async_id_symbol: kAsyncIdSymbol,
+    trigger_async_id_symbol: kTriggerAsyncIdSymbol,
+  },
+} = require('internal/async_hooks');
+const { isErrorStackTraceLimitWritable } = require('internal/errors');
+
 // *Must* match Environment::TickInfo::Fields in src/env.h.
 const kHasRejectionToWarn = 1;
 
-const maybeUnhandledPromises = new WeakMap();
+const maybeUnhandledPromises = new SafeWeakMap();
 const pendingUnhandledRejections = [];
 const asyncHandledRejections = [];
 let lastPromiseId = 0;
 
-// --unhandled-rejection=none:
+// --unhandled-rejections=none:
 // Emit 'unhandledRejection', but do not emit any warning.
 const kIgnoreUnhandledRejections = 0;
-// --unhandled-rejection=warn:
+
+// --unhandled-rejections=warn:
 // Emit 'unhandledRejection', then emit 'UnhandledPromiseRejectionWarning'.
 const kAlwaysWarnUnhandledRejections = 1;
-// --unhandled-rejection=strict:
+
+// --unhandled-rejections=strict:
 // Emit 'uncaughtException'. If it's not handled, print the error to stderr
 // and exit the process.
 // Otherwise, emit 'unhandledRejection'. If 'unhandledRejection' is not
 // handled, emit 'UnhandledPromiseRejectionWarning'.
-const kThrowUnhandledRejections = 2;
-// --unhandled-rejection is unset:
-// Emit 'unhandledRejection', if it's handled, emit
-// 'UnhandledPromiseRejectionWarning', then emit deprecation warning.
-const kDefaultUnhandledRejections = 3;
+const kStrictUnhandledRejections = 2;
+
+// --unhandled-rejections=throw:
+// Emit 'unhandledRejection', if it's unhandled, emit
+// 'uncaughtException'. If it's not handled, print the error to stderr
+// and exit the process.
+const kThrowUnhandledRejections = 3;
+
+// --unhandled-rejections=warn-with-error-code:
+// Emit 'unhandledRejection', if it's unhandled, emit
+// 'UnhandledPromiseRejectionWarning', then set process exit code to 1.
+
+const kWarnWithErrorCodeUnhandledRejections = 4;
 
 let unhandledRejectionsMode;
 
@@ -57,6 +82,12 @@ function hasRejectionToWarn() {
   return tickInfo[kHasRejectionToWarn] === 1;
 }
 
+function isErrorLike(o) {
+  return typeof o === 'object' &&
+         o !== null &&
+         ObjectPrototypeHasOwnProperty(o, 'stack');
+}
+
 function getUnhandledRejectionsMode() {
   const { getOptionValue } = require('internal/options');
   switch (getOptionValue('--unhandled-rejections')) {
@@ -65,9 +96,13 @@ function getUnhandledRejectionsMode() {
     case 'warn':
       return kAlwaysWarnUnhandledRejections;
     case 'strict':
+      return kStrictUnhandledRejections;
+    case 'throw':
       return kThrowUnhandledRejections;
+    case 'warn-with-error-code':
+      return kWarnWithErrorCodeUnhandledRejections;
     default:
-      return kDefaultUnhandledRejections;
+      return kThrowUnhandledRejections;
   }
 }
 
@@ -91,22 +126,38 @@ function promiseRejectHandler(type, promise, reason) {
   }
 }
 
+const multipleResolvesDeprecate = deprecate(
+  () => {},
+  'The multipleResolves event has been deprecated.',
+  'DEP0160',
+);
 function resolveError(type, promise, reason) {
   // We have to wrap this in a next tick. Otherwise the error could be caught by
   // the executed promise.
   process.nextTick(() => {
-    process.emit('multipleResolves', type, promise, reason);
+    if (process.emit('multipleResolves', type, promise, reason)) {
+      multipleResolvesDeprecate();
+    }
   });
 }
 
 function unhandledRejection(promise, reason) {
+  const emit = (reason, promise, promiseInfo) => {
+    if (promiseInfo.domain) {
+      return promiseInfo.domain.emit('error', reason);
+    }
+    return process.emit('unhandledRejection', reason, promise);
+  };
+
   maybeUnhandledPromises.set(promise, {
     reason,
     uid: ++lastPromiseId,
-    warned: false
+    warned: false,
+    domain: process.domain,
+    emit,
   });
   // This causes the promise to be referenced at least for one tick.
-  pendingUnhandledRejections.push(promise);
+  ArrayPrototypePush(pendingUnhandledRejections, promise);
   setHasRejectionToWarn(true);
 }
 
@@ -122,7 +173,7 @@ function handledRejection(promise) {
                                 `asynchronously (rejection id: ${uid})`);
       warning.name = 'PromiseRejectionHandledWarning';
       warning.id = uid;
-      asyncHandledRejections.push({ promise, warning });
+      ArrayPrototypePush(asyncHandledRejections, { promise, warning });
       setHasRejectionToWarn(true);
       return;
     }
@@ -141,30 +192,28 @@ function emitUnhandledRejectionWarning(uid, reason) {
       'To terminate the node process on unhandled promise ' +
       'rejection, use the CLI flag `--unhandled-rejections=strict` (see ' +
       'https://nodejs.org/api/cli.html#cli_unhandled_rejections_mode). ' +
-      `(rejection id: ${uid})`
+      `(rejection id: ${uid})`,
   );
   try {
-    if (reason instanceof Error) {
+    if (isErrorLike(reason)) {
       warning.stack = reason.stack;
       process.emitWarning(reason.stack, unhandledRejectionErrName);
     } else {
       process.emitWarning(
         noSideEffectsToString(reason), unhandledRejectionErrName);
     }
-  } catch {}
+  } catch {
+    try {
+      process.emitWarning(
+        noSideEffectsToString(reason), unhandledRejectionErrName);
+    } catch {
+      // Ignore.
+    }
+  }
 
   process.emitWarning(warning);
 }
 
-let deprecationWarned = false;
-function emitDeprecationWarning() {
-  process.emitWarning(
-    'Unhandled promise rejections are deprecated. In the future, ' +
-    'promise rejections that are not handled will terminate the ' +
-    'Node.js process with a non-zero exit code.',
-    'DeprecationWarning', 'DEP0018');
-}
-
 // If this method returns true, we've executed user code or triggered
 // a warning to be emitted which requires the microtask and next tick
 // queues to be drained again.
@@ -172,7 +221,7 @@ function processPromiseRejections() {
   let maybeScheduledTicksOrMicrotasks = asyncHandledRejections.length > 0;
 
   while (asyncHandledRejections.length > 0) {
-    const { promise, warning } = asyncHandledRejections.shift();
+    const { promise, warning } = ArrayPrototypeShift(asyncHandledRejections);
     if (!process.emit('rejectionHandled', promise)) {
       process.emitWarning(warning);
     }
@@ -180,41 +229,81 @@ function processPromiseRejections() {
 
   let len = pendingUnhandledRejections.length;
   while (len--) {
-    const promise = pendingUnhandledRejections.shift();
+    const promise = ArrayPrototypeShift(pendingUnhandledRejections);
     const promiseInfo = maybeUnhandledPromises.get(promise);
     if (promiseInfo === undefined) {
       continue;
     }
     promiseInfo.warned = true;
-    const { reason, uid } = promiseInfo;
-    switch (unhandledRejectionsMode) {
-      case kThrowUnhandledRejections: {
-        const err = reason instanceof Error ?
-          reason : generateUnhandledRejectionError(reason);
-        triggerUncaughtException(err, true /* fromPromise */);
-        const handled = process.emit('unhandledRejection', reason, promise);
-        if (!handled) emitUnhandledRejectionWarning(uid, reason);
-        break;
-      }
-      case kIgnoreUnhandledRejections: {
-        process.emit('unhandledRejection', reason, promise);
-        break;
-      }
-      case kAlwaysWarnUnhandledRejections: {
-        process.emit('unhandledRejection', reason, promise);
-        emitUnhandledRejectionWarning(uid, reason);
-        break;
-      }
-      case kDefaultUnhandledRejections: {
-        const handled = process.emit('unhandledRejection', reason, promise);
-        if (!handled) {
-          emitUnhandledRejectionWarning(uid, reason);
-          if (!deprecationWarned) {
-            emitDeprecationWarning();
-            deprecationWarned = true;
+    const { reason, uid, emit } = promiseInfo;
+
+    let needPop = true;
+    const {
+      [kAsyncIdSymbol]: promiseAsyncId,
+      [kTriggerAsyncIdSymbol]: promiseTriggerAsyncId,
+    } = promise;
+    // We need to check if async_hooks are enabled
+    // don't use enabledHooksExist as a Promise could
+    // come from a vm.* context and not have an async id
+    if (typeof promiseAsyncId !== 'undefined') {
+      pushAsyncContext(
+        promiseAsyncId,
+        promiseTriggerAsyncId,
+        promise,
+      );
+    }
+    try {
+      switch (unhandledRejectionsMode) {
+        case kStrictUnhandledRejections: {
+          const err = isErrorLike(reason) ?
+            reason : generateUnhandledRejectionError(reason);
+          // This destroys the async stack, don't clear it after
+          triggerUncaughtException(err, true /* fromPromise */);
+          if (typeof promiseAsyncId !== 'undefined') {
+            pushAsyncContext(
+              promise[kAsyncIdSymbol],
+              promise[kTriggerAsyncIdSymbol],
+              promise,
+            );
           }
+          const handled = emit(reason, promise, promiseInfo);
+          if (!handled) emitUnhandledRejectionWarning(uid, reason);
+          break;
+        }
+        case kIgnoreUnhandledRejections: {
+          emit(reason, promise, promiseInfo);
+          break;
+        }
+        case kAlwaysWarnUnhandledRejections: {
+          emit(reason, promise, promiseInfo);
+          emitUnhandledRejectionWarning(uid, reason);
+          break;
+        }
+        case kThrowUnhandledRejections: {
+          const handled = emit(reason, promise, promiseInfo);
+          if (!handled) {
+            const err = isErrorLike(reason) ?
+              reason : generateUnhandledRejectionError(reason);
+              // This destroys the async stack, don't clear it after
+            triggerUncaughtException(err, true /* fromPromise */);
+            needPop = false;
+          }
+          break;
+        }
+        case kWarnWithErrorCodeUnhandledRejections: {
+          const handled = emit(reason, promise, promiseInfo);
+          if (!handled) {
+            emitUnhandledRejectionWarning(uid, reason);
+            process.exitCode = 1;
+          }
+          break;
+        }
+      }
+    } finally {
+      if (needPop) {
+        if (typeof promiseAsyncId !== 'undefined') {
+          popAsyncContext(promiseAsyncId);
         }
-        break;
       }
     }
     maybeScheduledTicksOrMicrotasks = true;
@@ -226,11 +315,12 @@ function processPromiseRejections() {
 function getErrorWithoutStack(name, message) {
   // Reset the stack to prevent any overhead.
   const tmp = Error.stackTraceLimit;
-  Error.stackTraceLimit = 0;
+  if (isErrorStackTraceLimitWritable()) Error.stackTraceLimit = 0;
   // eslint-disable-next-line no-restricted-syntax
   const err = new Error(message);
-  Error.stackTraceLimit = tmp;
+  if (isErrorStackTraceLimitWritable()) Error.stackTraceLimit = tmp;
   ObjectDefineProperty(err, 'name', {
+    __proto__: null,
     value: name,
     enumerable: false,
     writable: true,
@@ -255,10 +345,9 @@ function generateUnhandledRejectionError(reason) {
 function listenForRejections() {
   setPromiseRejectCallback(promiseRejectHandler);
 }
-
 module.exports = {
   hasRejectionToWarn,
   setHasRejectionToWarn,
   listenForRejections,
-  processPromiseRejections
+  processPromiseRejections,
 };
