@@ -43,7 +43,6 @@
 'use strict';
 
 const {
-  ArrayPrototypeConcat,
   ArrayPrototypeFilter,
   ArrayPrototypeFindIndex,
   ArrayPrototypeForEach,
@@ -52,6 +51,7 @@ const {
   ArrayPrototypeMap,
   ArrayPrototypePop,
   ArrayPrototypePush,
+  ArrayPrototypePushApply,
   ArrayPrototypeReverse,
   ArrayPrototypeShift,
   ArrayPrototypeSlice,
@@ -62,6 +62,7 @@ const {
   Boolean,
   Error,
   FunctionPrototypeBind,
+  JSONStringify,
   MathMaxApply,
   NumberIsNaN,
   NumberParseFloat,
@@ -77,8 +78,6 @@ const {
   ReflectApply,
   RegExp,
   RegExpPrototypeExec,
-  RegExpPrototypeSymbolReplace,
-  RegExpPrototypeSymbolSplit,
   SafePromiseRace,
   SafeSet,
   SafeWeakSet,
@@ -99,19 +98,23 @@ const {
   globalThis,
 } = primordials;
 
-const { BuiltinModule } = require('internal/bootstrap/loaders');
+const { BuiltinModule } = require('internal/bootstrap/realm');
 const {
   makeRequireFunction,
-  addBuiltinLibsToObject
-} = require('internal/modules/cjs/helpers');
+  addBuiltinLibsToObject,
+} = require('internal/modules/helpers');
 const {
   isIdentifierStart,
-  isIdentifierChar
+  isIdentifierChar,
+  parse: acornParse,
 } = require('internal/deps/acorn/acorn/dist/acorn');
+const acornWalk = require('internal/deps/acorn/acorn-walk/dist/walk');
 const {
   decorateErrorStack,
   isError,
-  deprecate
+  deprecate,
+  SideEffectFreeRegExpPrototypeSymbolReplace,
+  SideEffectFreeRegExpPrototypeSymbolSplit,
 } = require('internal/util');
 const { inspect } = require('internal/util/inspect');
 const vm = require('vm');
@@ -119,9 +122,10 @@ const path = require('path');
 const fs = require('fs');
 const { Interface } = require('readline');
 const {
-  commonPrefix
+  commonPrefix,
 } = require('internal/readline/utils');
 const { Console } = require('console');
+const { shouldColorize } = require('internal/util/colors');
 const CJSModule = require('internal/modules/cjs/loader').Module;
 let _builtinLibs = ArrayPrototypeFilter(
   CJSModule.builtinModules,
@@ -154,7 +158,7 @@ const {
   validateObject,
 } = require('internal/validators');
 const experimentalREPLAwait = getOptionValue(
-  '--experimental-repl-await'
+  '--experimental-repl-await',
 );
 const pendingDeprecation = getOptionValue('--pending-deprecation');
 const {
@@ -174,7 +178,7 @@ const {
 } = internalBinding('util');
 const {
   startSigintWatchdog,
-  stopSigintWatchdog
+  stopSigintWatchdog,
 } = internalBinding('contextify');
 
 const history = require('internal/repl/history');
@@ -224,6 +228,28 @@ module.paths = CJSModule._nodeModulePaths(module.filename);
 const writer = (obj) => inspect(obj, writer.options);
 writer.options = { ...inspect.defaultOptions, showProxy: true };
 
+// Converts static import statement to dynamic import statement
+const toDynamicImport = (codeLine) => {
+  let dynamicImportStatement = '';
+  const ast = acornParse(codeLine, { __proto__: null, sourceType: 'module', ecmaVersion: 'latest' });
+  acornWalk.ancestor(ast, {
+    ImportDeclaration(node) {
+      const awaitDynamicImport = `await import(${JSONStringify(node.source.value)});`;
+      if (node.specifiers.length === 0) {
+        dynamicImportStatement += awaitDynamicImport;
+      } else if (node.specifiers.length === 1 && node.specifiers[0].type === 'ImportNamespaceSpecifier') {
+        dynamicImportStatement += `const ${node.specifiers[0].local.name} = ${awaitDynamicImport}`;
+      } else {
+        const importNames = ArrayPrototypeJoin(ArrayPrototypeMap(node.specifiers, ({ local, imported }) =>
+          (local.name === imported?.name ? local.name : `${imported?.name ?? 'default'}: ${local.name}`),
+        ), ', ');
+        dynamicImportStatement += `const { ${importNames} } = ${awaitDynamicImport}`;
+      }
+    },
+  });
+  return dynamicImportStatement;
+};
+
 function REPLServer(prompt,
                     stream,
                     eval_,
@@ -272,11 +298,7 @@ function REPLServer(prompt,
 
   if (options.terminal && options.useColors === undefined) {
     // If possible, check if stdout supports colors or not.
-    if (options.output.hasColors) {
-      options.useColors = options.output.hasColors();
-    } else if (process.env.NODE_DISABLE_COLORS === undefined) {
-      options.useColors = true;
-    }
+    options.useColors = shouldColorize(options.output) || process.env.NODE_DISABLE_COLORS === undefined;
   }
 
   // TODO(devsnek): Add a test case for custom eval functions.
@@ -298,7 +320,7 @@ function REPLServer(prompt,
                 'DEP0141') :
       (val) => this.input = val,
     enumerable: false,
-    configurable: true
+    configurable: true,
   });
   ObjectDefineProperty(this, 'outputStream', {
     __proto__: null,
@@ -315,7 +337,7 @@ function REPLServer(prompt,
                 'DEP0141') :
       (val) => this.output = val,
     enumerable: false,
-    configurable: true
+    configurable: true,
   });
 
   this.allowBlockingCompletions = !!options.allowBlockingCompletions;
@@ -456,15 +478,15 @@ function REPLServer(prompt,
 
           // Remove all "await"s and attempt running the script
           // in order to detect if error is truly non recoverable
-          const fallbackCode = RegExpPrototypeSymbolReplace(/\bawait\b/g, code, '');
+          const fallbackCode = SideEffectFreeRegExpPrototypeSymbolReplace(/\bawait\b/g, code, '');
           try {
             vm.createScript(fallbackCode, {
               filename: file,
               displayErrors: true,
-              importModuleDynamically: (specifier, _, importAssertions) => {
+              importModuleDynamically: (specifier, _, importAttributes) => {
                 return asyncESM.esmLoader.import(specifier, parentURL,
-                                                 importAssertions);
-              }
+                                                 importAttributes);
+              },
             });
           } catch (fallbackError) {
             if (isRecoverableError(fallbackError, fallbackCode)) {
@@ -505,10 +527,10 @@ function REPLServer(prompt,
           script = vm.createScript(code, {
             filename: file,
             displayErrors: true,
-            importModuleDynamically: (specifier, _, importAssertions) => {
+            importModuleDynamically: (specifier, _, importAttributes) => {
               return asyncESM.esmLoader.import(specifier, parentURL,
-                                               importAssertions);
-            }
+                                               importAttributes);
+            },
           });
         } catch (e) {
           debug('parse error %j', code, e);
@@ -565,7 +587,7 @@ function REPLServer(prompt,
         try {
           const scriptOptions = {
             displayErrors: false,
-            breakOnSigint: self.breakEvalOnSigint
+            breakOnSigint: self.breakEvalOnSigint,
           };
 
           if (self.useGlobal) {
@@ -654,7 +676,7 @@ function REPLServer(prompt,
           // find the first frame with a null function name
           const idx = ArrayPrototypeFindIndex(
             ArrayPrototypeReverse(stackFrames),
-            (frame) => frame.getFunctionName() === null
+            (frame) => frame.getFunctionName() === null,
           );
           // If found, get rid of it and everything below it
           frames = ArrayPrototypeSplice(stackFrames, idx + 1);
@@ -681,25 +703,25 @@ function REPLServer(prompt,
         if (e.stack) {
           if (e.name === 'SyntaxError') {
             // Remove stack trace.
-            e.stack = RegExpPrototypeSymbolReplace(
+            e.stack = SideEffectFreeRegExpPrototypeSymbolReplace(
               /^\s+at\s.*\n?/gm,
-              RegExpPrototypeSymbolReplace(/^REPL\d+:\d+\r?\n/, e.stack, ''),
+              SideEffectFreeRegExpPrototypeSymbolReplace(/^REPL\d+:\d+\r?\n/, e.stack, ''),
               '');
             const importErrorStr = 'Cannot use import statement outside a ' +
               'module';
             if (StringPrototypeIncludes(e.message, importErrorStr)) {
               e.message = 'Cannot use import statement inside the Node.js ' +
-                'REPL, alternatively use dynamic import';
-              e.stack = RegExpPrototypeSymbolReplace(
+                'REPL, alternatively use dynamic import: ' + toDynamicImport(self.lines.at(-1));
+              e.stack = SideEffectFreeRegExpPrototypeSymbolReplace(
                 /SyntaxError:.*\n/,
                 e.stack,
                 `SyntaxError: ${e.message}\n`);
             }
           } else if (self.replMode === module.exports.REPL_MODE_STRICT) {
-            e.stack = RegExpPrototypeSymbolReplace(
+            e.stack = SideEffectFreeRegExpPrototypeSymbolReplace(
               /(\s+at\s+REPL\d+:)(\d+)/,
               e.stack,
-              (_, pre, line) => pre + (line - 1)
+              (_, pre, line) => pre + (line - 1),
             );
           }
         }
@@ -728,7 +750,7 @@ function REPLServer(prompt,
       if (errStack === '') {
         errStack = self.writer(e);
       }
-      const lines = RegExpPrototypeSymbolSplit(/(?<=\n)/, errStack);
+      const lines = SideEffectFreeRegExpPrototypeSymbolSplit(/(?<=\n)/, errStack);
       let matched = false;
 
       errStack = '';
@@ -769,7 +791,7 @@ function REPLServer(prompt,
     completer: options.completer || completer,
     terminal: options.terminal,
     historySize: options.historySize,
-    prompt
+    prompt,
   }]);
 
   self.resetContext();
@@ -795,7 +817,7 @@ function REPLServer(prompt,
           return ObjectAssign(writer.options, options);
         },
         enumerable: true,
-        configurable: true
+        configurable: true,
       });
     }
   }
@@ -840,7 +862,7 @@ function REPLServer(prompt,
         return;
       }
       self.output.write(
-        '(To exit, press Ctrl+C again or Ctrl+D or type .exit)\n'
+        '(To exit, press Ctrl+C again or Ctrl+D or type .exit)\n',
       );
       sawSIGINT = true;
     } else {
@@ -969,12 +991,12 @@ function REPLServer(prompt,
 
   const {
     clearPreview,
-    showPreview
+    showPreview,
   } = setupPreview(
     this,
     kContextId,
     kBufferedCommandSymbol,
-    preview
+    preview,
   );
 
   // Wrap readline tty to enable editor mode and pausing.
@@ -995,9 +1017,8 @@ function REPLServer(prompt,
       clearPreview(key);
       if (!reverseSearch(d, key)) {
         ttyWrite(d, key);
-        if (key.name !== 'escape') {
-          showPreview();
-        }
+        const showCompletionPreview = key.name !== 'escape';
+        showPreview(showCompletionPreview);
       }
       return;
     }
@@ -1058,13 +1079,13 @@ REPLServer.prototype.close = function close() {
   if (this.terminal && this._flushing && !this._closingOnFlush) {
     this._closingOnFlush = true;
     this.once('flushHistory', () =>
-      ReflectApply(Interface.prototype.close, this, [])
+      ReflectApply(Interface.prototype.close, this, []),
     );
 
     return;
   }
   process.nextTick(() =>
-    ReflectApply(Interface.prototype.close, this, [])
+    ReflectApply(Interface.prototype.close, this, []),
   );
 };
 
@@ -1099,7 +1120,7 @@ REPLServer.prototype.createContext = function() {
       __proto__: null,
       configurable: true,
       writable: true,
-      value: _console
+      value: _console,
     });
   }
 
@@ -1110,13 +1131,13 @@ REPLServer.prototype.createContext = function() {
     __proto__: null,
     configurable: true,
     writable: true,
-    value: replModule
+    value: replModule,
   });
   ObjectDefineProperty(context, 'require', {
     __proto__: null,
     configurable: true,
     writable: true,
-    value: makeRequireFunction(replModule)
+    value: makeRequireFunction(replModule),
   });
 
   addBuiltinLibsToObject(context, '<REPL>');
@@ -1142,7 +1163,7 @@ REPLServer.prototype.resetContext = function() {
         this.underscoreAssigned = true;
         this.output.write('Expression assignment to _ now disabled.\n');
       }
-    }
+    },
   });
 
   ObjectDefineProperty(this.context, '_error', {
@@ -1156,7 +1177,7 @@ REPLServer.prototype.resetContext = function() {
         this.output.write(
           'Expression assignment to _error now disabled.\n');
       }
-    }
+    },
   });
 
   // Allow REPL extensions to extend the new context
@@ -1242,7 +1263,7 @@ function getGlobalLexicalScopeNames(contextId) {
   return sendInspectorCommand((session) => {
     let names = [];
     session.post('Runtime.globalLexicalScopeNames', {
-      executionContextId: contextId
+      executionContextId: contextId,
     }, (error, result) => {
       if (!error) names = result.names;
     });
@@ -1276,9 +1297,9 @@ function completeFSFunctions(match) {
   const completions = ArrayPrototypeMap(
     ArrayPrototypeFilter(
       fileList,
-      (dirent) => StringPrototypeStartsWith(dirent.name, baseName)
+      (dirent) => StringPrototypeStartsWith(dirent.name, baseName),
     ),
-    (d) => d.name
+    (d) => d.name,
   );
 
   return [[completions], baseName];
@@ -1333,7 +1354,9 @@ function complete(line, callback) {
       } else if (RegExpPrototypeExec(/^\.\.?\//, completeOn) !== null) {
         paths = [process.cwd()];
       } else {
-        paths = ArrayPrototypeConcat(module.paths, CJSModule.globalPaths);
+        paths = [];
+        ArrayPrototypePushApply(paths, module.paths);
+        ArrayPrototypePushApply(paths, CJSModule.globalPaths);
       }
 
       ArrayPrototypeForEach(paths, (dir) => {
@@ -1358,7 +1381,7 @@ function complete(line, callback) {
           const absolute = path.resolve(dir, dirent.name);
           if (ArrayPrototypeSome(
             gracefulReaddir(absolute) || [],
-            (subfile) => ArrayPrototypeIncludes(indexes, subfile)
+            (subfile) => ArrayPrototypeIncludes(indexes, subfile),
           )) {
             ArrayPrototypePush(group, `${subdir}${dirent.name}`);
           }
@@ -1553,7 +1576,7 @@ function complete(line, callback) {
           // behavior.
           return StringPrototypeStartsWith(
             StringPrototypeToLocaleLowerCase(str),
-            lowerCaseFilter
+            lowerCaseFilter,
           );
         });
         if (filteredGroup.length) {
@@ -1664,7 +1687,7 @@ function _memory(cmd) {
         // scope will not work for this function.
         ArrayPrototypePush(self.lines.level, {
           line: self.lines.length - 1,
-          depth: depth
+          depth: depth,
         });
       } else if (depth < 0) {
         // Going... up.
@@ -1714,7 +1737,7 @@ function defineDefaultCommands(repl) {
     action: function() {
       this.clearBufferedCommand();
       this.displayPrompt();
-    }
+    },
   });
 
   let clearMessage;
@@ -1732,14 +1755,14 @@ function defineDefaultCommands(repl) {
         this.resetContext();
       }
       this.displayPrompt();
-    }
+    },
   });
 
   repl.defineCommand('exit', {
     help: 'Exit the REPL',
     action: function() {
       this.close();
-    }
+    },
   });
 
   repl.defineCommand('help', {
@@ -1747,7 +1770,7 @@ function defineDefaultCommands(repl) {
     action: function() {
       const names = ArrayPrototypeSort(ObjectKeys(this.commands));
       const longestNameLength = MathMaxApply(
-        ArrayPrototypeMap(names, (name) => name.length)
+        ArrayPrototypeMap(names, (name) => name.length),
       );
       ArrayPrototypeForEach(names, (name) => {
         const cmd = this.commands[name];
@@ -1759,7 +1782,7 @@ function defineDefaultCommands(repl) {
       this.output.write('\nPress Ctrl+C to abort current expression, ' +
         'Ctrl+D to exit the REPL\n');
       this.displayPrompt();
-    }
+    },
   });
 
   repl.defineCommand('save', {
@@ -1772,7 +1795,7 @@ function defineDefaultCommands(repl) {
         this.output.write(`Failed to save: ${file}\n`);
       }
       this.displayPrompt();
-    }
+    },
   });
 
   repl.defineCommand('load', {
@@ -1788,14 +1811,14 @@ function defineDefaultCommands(repl) {
           this.write('\n');
         } else {
           this.output.write(
-            `Failed to load: ${file} is not a valid file\n`
+            `Failed to load: ${file} is not a valid file\n`,
           );
         }
       } catch {
         this.output.write(`Failed to load: ${file}\n`);
       }
       this.displayPrompt();
-    }
+    },
   });
   if (repl.terminal) {
     repl.defineCommand('editor', {
@@ -1804,7 +1827,7 @@ function defineDefaultCommands(repl) {
         _turnOnEditorMode(this);
         this.output.write(
           '// Entering editor mode (Ctrl+D to finish, Ctrl+C to cancel)\n');
-      }
+      },
     });
   }
 }
@@ -1821,7 +1844,7 @@ module.exports = {
   REPLServer,
   REPL_MODE_SLOPPY,
   REPL_MODE_STRICT,
-  Recoverable
+  Recoverable,
 };
 
 ObjectDefineProperty(module.exports, 'builtinModules', {
@@ -1829,7 +1852,7 @@ ObjectDefineProperty(module.exports, 'builtinModules', {
   get: () => _builtinLibs,
   set: (val) => _builtinLibs = val,
   enumerable: true,
-  configurable: true
+  configurable: true,
 });
 
 ObjectDefineProperty(module.exports, '_builtinLibs', {
@@ -1837,13 +1860,13 @@ ObjectDefineProperty(module.exports, '_builtinLibs', {
   get: pendingDeprecation ? deprecate(
     () => _builtinLibs,
     'repl._builtinLibs is deprecated. Check module.builtinModules instead',
-    'DEP0142'
+    'DEP0142',
   ) : () => _builtinLibs,
   set: pendingDeprecation ? deprecate(
     (val) => _builtinLibs = val,
     'repl._builtinLibs is deprecated. Check module.builtinModules instead',
-    'DEP0142'
+    'DEP0142',
   ) : (val) => _builtinLibs = val,
   enumerable: false,
-  configurable: true
+  configurable: true,
 });
