@@ -37,3 +37,12 @@ assert.throws(
                 { name: 'TypeError' });
   assert.strictEqual(globalThis.globalThis, globalThis);
 }
+
+// Ensure that we cannot override console properties.
+{
+  const { log } = console;
+
+  assert.throws(() => { console.log = null; },
+                { name: 'TypeError' });
+  assert.strictEqual(console.log, log);
+}
