@@ -9,11 +9,11 @@ const http2 = require('http2');
 const fs = require('fs');
 const path = require('path');
 
-// piping should work as expected with createWriteStream
+// Piping should work as expected with createWriteStream
 
 const tmpdir = require('../common/tmpdir');
 tmpdir.refresh();
-const loc = fixtures.path('url-tests.js');
+const loc = fixtures.path('person-large.jpg');
 const fn = path.join(tmpdir.path, 'http2-url-tests.js');
 
 const server = http2.createServer();
