@@ -15,6 +15,9 @@ common.skipIfDumbTerminal();
 const tmpdir = require('../common/tmpdir');
 tmpdir.refresh();
 
+process.throwDeprecation = true;
+process.on('warning', common.mustNotCall());
+
 const defaultHistoryPath = path.join(tmpdir.path, '.node_repl_history');
 
 // Create an input stream specialized for testing an array of actions
@@ -57,6 +60,7 @@ const WORD_RIGHT = { name: 'right', ctrl: true };
 const GO_TO_END = { name: 'end' };
 const DELETE_WORD_LEFT = { name: 'backspace', ctrl: true };
 const SIGINT = { name: 'c', ctrl: true };
+const ESCAPE = { name: 'escape', meta: true };
 
 const prompt = '> ';
 const WAIT = '€';
@@ -70,6 +74,7 @@ const tests = [
     env: { NODE_REPL_HISTORY: defaultHistoryPath },
     test: [ 'let ab = 45', ENTER,
             '555 + 909', ENTER,
+            'let autocompleteMe = 123', ENTER,
             '{key : {key2 :[] }}', ENTER,
             'Array(100).fill(1).map((e, i) => i ** i)', LEFT, LEFT, DELETE,
             '2', ENTER],
@@ -78,7 +83,7 @@ const tests = [
   },
   {
     env: { NODE_REPL_HISTORY: defaultHistoryPath },
-    test: [UP, UP, UP, UP, UP, DOWN, DOWN, DOWN, DOWN, DOWN],
+    test: [UP, UP, UP, UP, UP, UP, DOWN, DOWN, DOWN, DOWN, DOWN, DOWN],
     expected: [prompt,
                `${prompt}Array(100).fill(1).map((e, i) => i ** 2)`,
                prev && '\n// [ 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100, 121, ' +
@@ -88,6 +93,7 @@ const tests = [
                  ' 2025, 2116, 2209,...',
                `${prompt}{key : {key2 :[] }}`,
                prev && '\n// { key: { key2: [] } }',
+               `${prompt}let autocompleteMe = 123`,
                `${prompt}555 + 909`,
                prev && '\n// 1464',
                `${prompt}let ab = 45`,
@@ -95,6 +101,7 @@ const tests = [
                `${prompt}let ab = 45`,
                `${prompt}555 + 909`,
                prev && '\n// 1464',
+               `${prompt}let autocompleteMe = 123`,
                `${prompt}{key : {key2 :[] }}`,
                prev && '\n// { key: { key2: [] } }',
                `${prompt}Array(100).fill(1).map((e, i) => i ** 2)`,
@@ -113,7 +120,7 @@ const tests = [
       'const foo = true', ENTER,
       '555n + 111n', ENTER,
       '5 + 5', ENTER,
-      '55 - 13 === 42', ENTER
+      '55 - 13 === 42', ENTER,
     ],
     expected: [],
     clean: false
@@ -124,7 +131,7 @@ const tests = [
     preview: false,
     showEscapeCodes: true,
     test: [
-      '55', UP, UP, UP, UP, UP, UP, ENTER
+      '55', UP, UP, UP, UP, UP, UP, UP, ENTER,
     ],
     expected: [
       '\x1B[1G', '\x1B[0J', prompt, '\x1B[3G',
@@ -147,7 +154,7 @@ const tests = [
       '\r\n', '55\n',
       '\x1B[1G', '\x1B[0J',
       '> ', '\x1B[3G',
-      '\r\n'
+      '\r\n',
     ],
     clean: true
   },
@@ -181,10 +188,12 @@ const tests = [
       ENTER,
       'veryLongName'.repeat(30),
       ENTER,
-      `${'\x1B[90m \x1B[39m'.repeat(235)} fun`,
+      `${'\x1B[90m \x1B[39m'.repeat(229)} aut`,
+      ESCAPE,
+      ENTER,
+      `${' '.repeat(230)} aut`,
+      ESCAPE,
       ENTER,
-      `${' '.repeat(236)} fun`,
-      ENTER
     ],
     expected: [],
     clean: false
@@ -215,7 +224,7 @@ const tests = [
       '2',
       BACKSPACE,
       '3',
-      SIGINT
+      SIGINT,
     ],
     // A = Cursor n up
     // B = Cursor n down
@@ -230,19 +239,20 @@ const tests = [
       prompt, '\x1B[3G',
       // 1. UP
       // This exceeds the maximum columns (250):
-      // Whitespace + prompt + ' // '.length + 'function'.length
-      // 236 + 2 + 4 + 8
+      // Whitespace + prompt + ' // '.length + 'autocompleteMe'.length
+      // 230 + 2 + 4 + 14
       '\x1B[1G', '\x1B[0J',
-      `${prompt}${' '.repeat(236)} fun`, '\x1B[243G',
-      ' // ction', '\x1B[243G',
-      ' // ction', '\x1B[243G',
+      `${prompt}${' '.repeat(230)} aut`, '\x1B[237G',
+      ' // ocompleteMe', '\x1B[237G',
+      '\n// 123', '\x1B[237G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
       '\x1B[0K',
       // 2. UP
       '\x1B[1G', '\x1B[0J',
-      `${prompt}${' '.repeat(235)} fun`, '\x1B[242G',
-      // TODO(BridgeAR): Investigate why the preview is generated twice.
-      ' // ction', '\x1B[242G',
-      ' // ction', '\x1B[242G',
+      `${prompt}${' '.repeat(229)} aut`, '\x1B[236G',
+      ' // ocompleteMe', '\x1B[236G',
+      '\n// 123', '\x1B[236G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
       // Preview cleanup
       '\x1B[0K',
       // 3. UP
@@ -310,7 +320,7 @@ const tests = [
       '\r\n',
       '\x1B[1G', '\x1B[0J',
       '> ', '\x1B[3G',
-      '\r\n'
+      '\r\n',
     ],
     clean: true
   },
@@ -318,9 +328,10 @@ const tests = [
     env: { NODE_REPL_HISTORY: defaultHistoryPath },
     showEscapeCodes: true,
     skip: !process.features.inspector,
+    checkTotal: true,
     test: [
-      'fu',
-      'n',
+      'au',
+      't',
       RIGHT,
       BACKSPACE,
       LEFT,
@@ -331,7 +342,13 @@ const tests = [
       BACKSPACE,
       WORD_LEFT,
       WORD_RIGHT,
-      ENTER
+      ESCAPE,
+      ENTER,
+      UP,
+      LEFT,
+      ENTER,
+      UP,
+      ENTER,
     ],
     // C = Cursor n forward
     // D = Cursor n back
@@ -340,51 +357,94 @@ const tests = [
     // K = Erase in line; 0 = right; 1 = left; 2 = total
     expected: [
       // 0.
-      // 'f'
-      '\x1B[1G', '\x1B[0J', prompt, '\x1B[3G', 'f',
+      // 'a'
+      '\x1B[1G', '\x1B[0J', prompt, '\x1B[3G', 'a',
       // 'u'
-      'u', ' // nction', '\x1B[5G',
-      // 'n' - Cleanup
+      'u', ' // tocompleteMe', '\x1B[5G',
+      '\n// 123', '\x1B[5G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
+      // 't' - Cleanup
       '\x1B[0K',
-      'n', ' // ction', '\x1B[6G',
+      't', ' // ocompleteMe', '\x1B[6G',
+      '\n// 123', '\x1B[6G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
       // 1. Right. Cleanup
       '\x1B[0K',
-      'ction',
+      'ocompleteMe',
+      '\n// 123', '\x1B[17G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
       // 2. Backspace. Refresh
-      '\x1B[1G', '\x1B[0J', `${prompt}functio`, '\x1B[10G',
+      '\x1B[1G', '\x1B[0J', `${prompt}autocompleteM`, '\x1B[16G',
       // Autocomplete and refresh?
-      ' // n', '\x1B[10G', ' // n', '\x1B[10G',
+      ' // e', '\x1B[16G',
+      '\n// 123', '\x1B[16G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
       // 3. Left. Cleanup
       '\x1B[0K',
-      '\x1B[1D', '\x1B[10G', ' // n', '\x1B[9G',
+      '\x1B[1D', '\x1B[16G', ' // e', '\x1B[15G',
       // 4. Left. Cleanup
-      '\x1B[10G', '\x1B[0K', '\x1B[9G',
-      '\x1B[1D', '\x1B[10G', ' // n', '\x1B[8G',
+      '\x1B[16G', '\x1B[0K', '\x1B[15G',
+      '\x1B[1D', '\x1B[16G', ' // e', '\x1B[14G',
       // 5. 'A' - Cleanup
-      '\x1B[10G', '\x1B[0K', '\x1B[8G',
+      '\x1B[16G', '\x1B[0K', '\x1B[14G',
       // Refresh
-      '\x1B[1G', '\x1B[0J', `${prompt}functAio`, '\x1B[9G',
+      '\x1B[1G', '\x1B[0J', `${prompt}autocompletAeM`, '\x1B[15G',
       // 6. Backspace. Refresh
-      '\x1B[1G', '\x1B[0J', `${prompt}functio`, '\x1B[8G', '\x1B[10G', ' // n',
-      '\x1B[8G', '\x1B[10G', ' // n',
-      '\x1B[8G', '\x1B[10G',
+      '\x1B[1G', '\x1B[0J', `${prompt}autocompleteM`,
+      '\x1B[14G', '\x1B[16G', ' // e',
+      '\x1B[14G', '\x1B[16G', ' // e',
+      '\x1B[14G', '\x1B[16G',
       // 7. Go to end. Cleanup
-      '\x1B[0K', '\x1B[8G', '\x1B[2C',
-      'n',
+      '\x1B[0K', '\x1B[14G', '\x1B[2C',
+      'e',
+      '\n// 123', '\x1B[17G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
       // 8. Backspace. Refresh
-      '\x1B[1G', '\x1B[0J', `${prompt}functio`, '\x1B[10G',
+      '\x1B[1G', '\x1B[0J', `${prompt}autocompleteM`, '\x1B[16G',
       // Autocomplete
-      ' // n', '\x1B[10G', ' // n', '\x1B[10G',
+      ' // e', '\x1B[16G',
+      '\n// 123', '\x1B[16G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
       // 9. Word left. Cleanup
-      '\x1B[0K', '\x1B[7D', '\x1B[10G', ' // n', '\x1B[3G', '\x1B[10G',
+      '\x1B[0K', '\x1B[13D', '\x1B[16G', ' // e', '\x1B[3G', '\x1B[16G',
       // 10. Word right. Cleanup
-      '\x1B[0K', '\x1B[3G', '\x1B[7C', ' // n', '\x1B[10G',
+      '\x1B[0K', '\x1B[3G', '\x1B[13C', ' // e', '\x1B[16G',
+      '\n// 123', '\x1B[16G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
+      // 11. ESCAPE
       '\x1B[0K',
-      // 11. ENTER
+      // 12. ENTER
       '\r\n',
-      'Uncaught ReferenceError: functio is not defined\n',
+      'Uncaught ReferenceError: autocompleteM is not defined\n',
       '\x1B[1G', '\x1B[0J',
-      prompt, '\x1B[3G', '\r\n'
+      // 13. UP
+      prompt, '\x1B[3G', '\x1B[1G', '\x1B[0J',
+      `${prompt}autocompleteM`, '\x1B[16G',
+      ' // e', '\x1B[16G',
+      '\n// 123', '\x1B[16G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
+      // 14. LEFT
+      '\x1B[0K', '\x1B[1D', '\x1B[16G',
+      ' // e', '\x1B[15G', '\x1B[16G',
+      // 15. ENTER
+      '\x1B[0K', '\x1B[15G', '\x1B[1C',
+      '\r\n',
+      'Uncaught ReferenceError: autocompleteM is not defined\n',
+      '\x1B[1G', '\x1B[0J',
+      prompt, '\x1B[3G',
+      // 16. UP
+      '\x1B[1G', '\x1B[0J',
+      `${prompt}autocompleteM`, '\x1B[16G',
+      ' // e', '\x1B[16G',
+      '\n// 123', '\x1B[16G',
+      '\x1B[1A', '\x1B[1B', '\x1B[2K', '\x1B[1A',
+      '\x1B[0K',
+      // 17. ENTER
+      'e', '\r\n',
+      '123\n',
+      '\x1B[1G', '\x1B[0J',
+      prompt, '\x1B[3G',
+      '\r\n',
     ],
     clean: true
   },
@@ -394,7 +454,7 @@ const tests = [
     skip: !process.features.inspector,
     test: [
       'util.inspect.replDefaults.showHidden',
-      ENTER
+      ENTER,
     ],
     expected: [],
     clean: false
@@ -411,7 +471,7 @@ const tests = [
       ' = true',
       ENTER,
       '[ ]',
-      ENTER
+      ENTER,
     ],
     expected: [
       prompt,
@@ -451,7 +511,7 @@ const tests = [
       WAIT, // The second call is not awaited. It won't trigger the preview.
       BACKSPACE,
       's',
-      BACKSPACE
+      BACKSPACE,
     ],
     expected: [
       prompt,
@@ -468,7 +528,110 @@ const tests = [
       prompt,
     ],
     clean: true
-  }
+  },
+  {
+    env: { NODE_REPL_HISTORY: defaultHistoryPath },
+    test: (function*() {
+      // Deleting Array iterator should not break history feature.
+      //
+      // Using a generator function instead of an object to allow the test to
+      // keep iterating even when Array.prototype[Symbol.iterator] has been
+      // deleted.
+      yield 'const ArrayIteratorPrototype =';
+      yield '  Object.getPrototypeOf(Array.prototype[Symbol.iterator]());';
+      yield ENTER;
+      yield 'const {next} = ArrayIteratorPrototype;';
+      yield ENTER;
+      yield 'const realArrayIterator = Array.prototype[Symbol.iterator];';
+      yield ENTER;
+      yield 'delete Array.prototype[Symbol.iterator];';
+      yield ENTER;
+      yield 'delete ArrayIteratorPrototype.next;';
+      yield ENTER;
+      yield UP;
+      yield UP;
+      yield DOWN;
+      yield DOWN;
+      yield 'fu';
+      yield 'n';
+      yield RIGHT;
+      yield BACKSPACE;
+      yield LEFT;
+      yield LEFT;
+      yield 'A';
+      yield BACKSPACE;
+      yield GO_TO_END;
+      yield BACKSPACE;
+      yield WORD_LEFT;
+      yield WORD_RIGHT;
+      yield ESCAPE;
+      yield ENTER;
+      yield 'Array.proto';
+      yield RIGHT;
+      yield '.pu';
+      yield ENTER;
+      yield 'ArrayIteratorPrototype.next = next;';
+      yield ENTER;
+      yield 'Array.prototype[Symbol.iterator] = realArrayIterator;';
+      yield ENTER;
+    })(),
+    expected: [],
+    clean: false
+  },
+  {
+    env: { NODE_REPL_HISTORY: defaultHistoryPath },
+    test: ['const util = {}', ENTER,
+           'ut', RIGHT, ENTER],
+    expected: [
+      prompt, ...'const util = {}',
+      'undefined\n',
+      prompt, ...'ut', ...(prev ? [' // il', '\n// {}',
+                                   'il', '\n// {}'] : [' // il', 'il']),
+      '{}\n',
+      prompt,
+    ],
+    clean: false
+  },
+  {
+    env: { NODE_REPL_HISTORY: defaultHistoryPath },
+    test: [
+      'const utilDesc = Reflect.getOwnPropertyDescriptor(globalThis, "util")',
+      ENTER,
+      'globalThis.util = {}', ENTER,
+      'ut', RIGHT, ENTER,
+      'Reflect.defineProperty(globalThis, "util", utilDesc)', ENTER],
+    expected: [
+      prompt, ...'const utilDesc = ' +
+      'Reflect.getOwnPropertyDescriptor(globalThis, "util")',
+      'undefined\n',
+      prompt, ...'globalThis.util = {}',
+      '{}\n',
+      prompt, ...'ut', ' // il', 'il',
+      '{}\n',
+      prompt, ...'Reflect.defineProperty(globalThis, "util", utilDesc)',
+      'true\n',
+      prompt,
+    ],
+    clean: false
+  },
+  {
+    // Test that preview should not be removed when pressing ESCAPE key
+    env: { NODE_REPL_HISTORY: defaultHistoryPath },
+    skip: !process.features.inspector,
+    test: [
+      '1+1',
+      ESCAPE,
+      ENTER,
+    ],
+    expected: [
+      prompt, ...'1+1',
+      '\n// 2',
+      '\n// 2',
+      '2\n',
+      prompt,
+    ],
+    clean: false
+  },
 ];
 const numtests = tests.length;
 
