@@ -4,8 +4,7 @@ const assert = require('assert');
 const cp = require('child_process');
 const fixtures = require('../common/fixtures');
 
-const args = ['--interactive', '--experimental-repl-await'];
-
+const args = ['--interactive'];
 const opts = { cwd: fixtures.path('es-modules') };
 const child = cp.spawn(process.execPath, args, opts);
 
@@ -17,7 +16,10 @@ child.stdout.on('data', (data) => {
 
 child.on('exit', common.mustCall(() => {
   const results = output.replace(/^> /mg, '').split('\n').slice(2);
-  assert.deepStrictEqual(results, ['[Module] { message: \'A message\' }', '']);
+  assert.deepStrictEqual(
+    results,
+    ['[Module: null prototype] { message: \'A message\' }', '']
+  );
 }));
 
 child.stdin.write('await import(\'./message.mjs\');\n');
