@@ -32,10 +32,10 @@ const message = 'Read, Eval, Print Loop';
 const prompt_unix = 'node via Unix socket> ';
 const prompt_tcp = 'node via TCP socket> ';
 
-// absolute path to test/fixtures/a.js
+// Absolute path to test/fixtures/a.js
 const moduleFilename = fixtures.path('a');
 
-// function for REPL to run
+// Function for REPL to run
 global.invoke_me = function(arg) {
   return `invoked ${arg}`;
 };
@@ -48,14 +48,14 @@ async function runReplTests(socket, prompt, tests) {
   let lineBuffer = '';
 
   for (const { send, expect } of tests) {
-    // expect can be a single line or multiple lines
+    // Expect can be a single line or multiple lines
     const expectedLines = Array.isArray(expect) ? expect : [ expect ];
 
     console.error('out:', JSON.stringify(send));
     socket.write(`${send}\n`);
 
     for (let expectedLine of expectedLines) {
-      // special value: kSource refers to last sent source text
+      // Special value: kSource refers to last sent source text
       if (expectedLine === kSource)
         expectedLine = send;
 
@@ -86,12 +86,11 @@ async function runReplTests(socket, prompt, tests) {
 
       console.error('in:', JSON.stringify(actualLine));
 
-      // Match a string directly, or a RegExp through .test().
+      // Match a string directly, or a RegExp.
       if (typeof expectedLine === 'string') {
         assert.strictEqual(actualLine, expectedLine);
       } else {
-        assert(expectedLine.test(actualLine),
-               `${actualLine} match ${expectedLine}`);
+        assert.match(actualLine, expectedLine);
       }
     }
   }
@@ -120,25 +119,25 @@ const unixTests = [
   {
     send: '{a:1}',
     expect: '{ a: 1 }'
-  }
+  },
 ];
 
 const strictModeTests = [
   {
     send: 'ref = 1',
-    expect: ['Thrown:', /^ReferenceError:\s/]
-  }
+    expect: [/^Uncaught ReferenceError:\s/]
+  },
 ];
 
 const errorTests = [
   // Uncaught error throws and prints out
   {
     send: 'throw new Error(\'test error\');',
-    expect: ['Thrown:', 'Error: test error']
+    expect: ['Uncaught Error: test error']
   },
   {
     send: "throw { foo: 'bar' };",
-    expect: "Thrown: { foo: 'bar' }"
+    expect: "Uncaught { foo: 'bar' }"
   },
   // Common syntax error is treated as multiline command
   {
@@ -153,7 +152,7 @@ const errorTests = [
   // But passing the same string to eval() should throw
   {
     send: 'eval("function test_func() {")',
-    expect: ['Thrown:', /^SyntaxError: /]
+    expect: [/^Uncaught SyntaxError: /]
   },
   // Can handle multiline template literals
   {
@@ -206,94 +205,94 @@ const errorTests = [
     send: 'JSON.parse(\'{"valid": "json"}\');',
     expect: '{ valid: \'json\' }'
   },
-  // invalid input to JSON.parse error is special case of syntax error,
+  // Invalid input to JSON.parse error is special case of syntax error,
   // should throw
   {
     send: 'JSON.parse(\'{invalid: \\\'json\\\'}\');',
-    expect: ['Thrown:', /^SyntaxError: /]
+    expect: [/^Uncaught SyntaxError: /]
   },
-  // end of input to JSON.parse error is special case of syntax error,
+  // End of input to JSON.parse error is special case of syntax error,
   // should throw
   {
     send: 'JSON.parse(\'066\');',
-    expect: ['Thrown:', /^SyntaxError: /]
+    expect: [/^Uncaught SyntaxError: /]
   },
   // should throw
   {
     send: 'JSON.parse(\'{\');',
-    expect: ['Thrown:', /^SyntaxError: /]
+    expect: [/^Uncaught SyntaxError: /]
   },
   // invalid RegExps are a special case of syntax error,
   // should throw
   {
     send: '/(/;',
-    expect: ['Thrown:', /^SyntaxError: /]
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      /^Uncaught SyntaxError: /,
+    ]
   },
   // invalid RegExp modifiers are a special case of syntax error,
   // should throw (GH-4012)
   {
     send: 'new RegExp("foo", "wrong modifier");',
-    expect: ['Thrown:', /^SyntaxError: /]
+    expect: [/^Uncaught SyntaxError: /]
   },
-  // strict mode syntax errors should be caught (GH-5178)
+  // Strict mode syntax errors should be caught (GH-5178)
   {
     send: '(function() { "use strict"; return 0755; })()',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
     send: '(function(a, a, b) { "use strict"; return a + b + c; })()',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
     send: '(function() { "use strict"; with (this) {} })()',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
     send: '(function() { "use strict"; var x; delete x; })()',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
     send: '(function() { "use strict"; eval = 17; })()',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
     send: '(function() { "use strict"; if (true) function f() { } })()',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      'Uncaught:',
+      /^SyntaxError: /,
     ]
   },
   // Named functions can be used:
@@ -308,7 +307,7 @@ const errorTests = [
   // Functions should not evaluate twice (#2773)
   {
     send: 'var I = [1,2,3,function() {}]; I.pop()',
-    expect: '[Function]'
+    expect: '[Function (anonymous)]'
   },
   // Multiline object
   {
@@ -328,6 +327,21 @@ const errorTests = [
     send: '1 }',
     expect: '{ a: 1 }'
   },
+  // Multiline class with private member.
+  {
+    send: 'class Foo { #private = true ',
+    expect: '... '
+  },
+  // Class field with bigint.
+  {
+    send: 'num = 123456789n',
+    expect: '... '
+  },
+  // Static class features.
+  {
+    send: 'static foo = "bar" }',
+    expect: 'undefined'
+  },
   // Multiline anonymous function with comment
   {
     send: '(function() {',
@@ -338,12 +352,12 @@ const errorTests = [
     expect: '... '
   },
   {
-    send: 'return 1;',
+    send: 'return 1n;',
     expect: '... '
   },
   {
     send: '})()',
-    expect: '1'
+    expect: '1n'
   },
   // Multiline function call
   {
@@ -358,12 +372,12 @@ const errorTests = [
     send: ')',
     expect: 'undefined'
   },
-  // npm prompt error message
+  // `npm` prompt error message.
   {
     send: 'npm install foobar',
     expect: [
-      'npm should be run outside of the node repl, in your normal shell.',
-      '(Press Control-D to exit.)'
+      'npm should be run outside of the Node.js REPL, in your normal shell.',
+      '(Press Ctrl+D to exit.)',
     ]
   },
   {
@@ -382,30 +396,29 @@ const errorTests = [
     send: 'var path = 42; path',
     expect: '42'
   },
-  // this makes sure that we don't print `undefined` when we actually print
+  // This makes sure that we don't print `undefined` when we actually print
   // the error message
   {
     send: '.invalid_repl_command',
     expect: 'Invalid REPL keyword'
   },
-  // this makes sure that we don't crash when we use an inherited property as
+  // This makes sure that we don't crash when we use an inherited property as
   // a REPL command
   {
     send: '.toString',
     expect: 'Invalid REPL keyword'
   },
-  // fail when we are not inside a String and a line continuation is used
+  // Fail when we are not inside a String and a line continuation is used
   {
     send: '[] \\',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
-  // do not fail when a String is created with line continuation
+  // Do not fail when a String is created with line continuation
   {
     send: '\'the\\\nfourth\\\neye\'',
     expect: ['... ... \'thefourtheye\'']
@@ -418,12 +431,12 @@ const errorTests = [
     send: '  \t    .break  \t  ',
     expect: ''
   },
-  // multiline strings preserve whitespace characters in them
+  // Multiline strings preserve whitespace characters in them
   {
     send: '\'the \\\n   fourth\t\t\\\n  eye  \'',
     expect: '... ... \'the    fourth\\t\\t  eye  \''
   },
-  // more than one multiline strings also should preserve whitespace chars
+  // More than one multiline strings also should preserve whitespace chars
   {
     send: '\'the \\\n   fourth\' +  \'\t\t\\\n  eye  \'',
     expect: '... ... \'the    fourth\\t\\t  eye  \''
@@ -433,26 +446,49 @@ const errorTests = [
     send: '\'\\\n.break',
     expect: '... ' + prompt_unix
   },
-  // using REPL command "help" within a string literal should still work
+  // Using REPL command "help" within a string literal should still work
   {
     send: '\'thefourth\\\n.help\neye\'',
     expect: [
       /\.break/,
       /\.clear/,
-      /\.editor/,
       /\.exit/,
       /\.help/,
       /\.load/,
       /\.save/,
-      /'thefourtheye'/
+      '',
+      'Press Ctrl+C to abort current expression, Ctrl+D to exit the REPL',
+      /'thefourtheye'/,
     ]
   },
-  // empty lines in the REPL should be allowed
+  // Check for wrapped objects.
+  {
+    send: '{ a: 1 }.a', // ({ a: 1 }.a);
+    expect: '1'
+  },
+  {
+    send: '{ a: 1 }.a;', // { a: 1 }.a;
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      /^Uncaught SyntaxError: /,
+    ]
+  },
+  {
+    send: '{ a: 1 }["a"] === 1', // ({ a: 1 }['a'] === 1);
+    expect: 'true'
+  },
+  {
+    send: '{ a: 1 }["a"] === 1;', // { a: 1 }; ['a'] === 1;
+    expect: 'false'
+  },
+  // Empty lines in the REPL should be allowed
   {
     send: '\n\r\n\r\n',
     expect: ''
   },
-  // empty lines in the string literals should not affect the string
+  // Empty lines in the string literals should not affect the string
   {
     send: '\'the\\\n\\\nfourtheye\'\n',
     expect: '... ... \'thefourtheye\''
@@ -462,14 +498,14 @@ const errorTests = [
     send: '/(.)(.)(.)(.)(.)(.)(.)(.)(.)/.test(\'123456789\')\n',
     expect: 'true'
   },
-  // the following test's result depends on the RegExp's match from the above
+  // The following test's result depends on the RegExp's match from the above
   {
     send: 'RegExp.$1\nRegExp.$2\nRegExp.$3\nRegExp.$4\nRegExp.$5\n' +
           'RegExp.$6\nRegExp.$7\nRegExp.$8\nRegExp.$9\n',
     expect: ['\'1\'', '\'2\'', '\'3\'', '\'4\'', '\'5\'', '\'6\'',
              '\'7\'', '\'8\'', '\'9\'']
   },
-  // regression tests for https://github.com/nodejs/node/issues/2749
+  // Regression tests for https://github.com/nodejs/node/issues/2749
   {
     send: 'function x() {\nreturn \'\\n\';\n }',
     expect: '... ... undefined'
@@ -478,7 +514,7 @@ const errorTests = [
     send: 'function x() {\nreturn \'\\\\\';\n }',
     expect: '... ... undefined'
   },
-  // regression tests for https://github.com/nodejs/node/issues/3421
+  // Regression tests for https://github.com/nodejs/node/issues/3421
   {
     send: 'function x() {\n//\'\n }',
     expect: '... ... undefined'
@@ -528,16 +564,20 @@ const errorTests = [
     expect: '... ... ... undefined'
   },
   // REPL should get a normal require() function, not one that allows
-  // access to internal modules without the --expose_internals flag.
+  // access to internal modules without the --expose-internals flag.
   {
     send: 'require("internal/repl")',
     expect: [
-      'Thrown:',
-      /^{ Error: Cannot find module 'internal\/repl'/,
-      /^    at .*/,
-      /^    at .*/,
-      /^    at .*/,
-      /^    at .*/
+      /^Uncaught Error: Cannot find module 'internal\/repl'/,
+      /^Require stack:/,
+      /^- <repl>/,
+      /^ {4}at .*/,
+      /^ {4}at .*/,
+      /^ {4}at .*/,
+      /^ {4}at .*/,
+      "  code: 'MODULE_NOT_FOUND',",
+      "  requireStack: [ '<repl>' ]",
+      '}',
     ]
   },
   // REPL should handle quotes within regexp literal in multiline mode
@@ -566,11 +606,10 @@ const errorTests = [
   {
     send: 'a = 3.5e',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   // Mitigate https://github.com/nodejs/node/issues/548
@@ -586,22 +625,20 @@ const errorTests = [
   {
     send: 'a = 3.5e',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   // Avoid emitting stack trace
   {
     send: 'a = 3.5e',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
 
@@ -666,14 +703,13 @@ const errorTests = [
   {
     send: '...[]',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
-  // bring back the repl to prompt
+  // Bring back the repl to prompt
   {
     send: '.break',
     expect: ''
@@ -681,31 +717,28 @@ const errorTests = [
   {
     send: 'console.log("Missing comma in arg list" process.version)',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
     send: 'x = {\nfield\n{',
     expect: [
-      '... ... Thrown:',
-      '{',
+      '... ... {',
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
     send: '(2 + 3))',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
     ]
   },
   {
@@ -719,11 +752,43 @@ const errorTests = [
   {
     send: '} else {',
     expect: [
-      'Thrown:',
       kSource,
       kArrow,
       '',
-      /^SyntaxError: /
+      /^Uncaught SyntaxError: /,
+    ]
+  },
+  {
+    send: 'console',
+    expect: [
+      'Object [console] {',
+      '  log: [Function: log],',
+      '  warn: [Function: warn],',
+      '  dir: [Function: dir],',
+      '  time: [Function: time],',
+      '  timeEnd: [Function: timeEnd],',
+      '  timeLog: [Function: timeLog],',
+      '  trace: [Function: trace],',
+      '  assert: [Function: assert],',
+      '  clear: [Function: clear],',
+      '  count: [Function: count],',
+      '  countReset: [Function: countReset],',
+      '  group: [Function: group],',
+      '  groupEnd: [Function: groupEnd],',
+      '  table: [Function: table],',
+      / {2}debug: \[Function: (debug|log)],/,
+      / {2}info: \[Function: (info|log)],/,
+      / {2}dirxml: \[Function: (dirxml|log)],/,
+      / {2}error: \[Function: (error|warn)],/,
+      / {2}groupCollapsed: \[Function: (groupCollapsed|group)],/,
+      / {2}Console: \[Function: Console],?/,
+      ...process.features.inspector ? [
+        '  profile: [Function: profile],',
+        '  profileEnd: [Function: profileEnd],',
+        '  timeStamp: [Function: timeStamp],',
+        '  context: [Function: context]',
+      ] : [],
+      '}',
     ]
   },
 ];
@@ -744,7 +809,84 @@ const tcpTests = [
   {
     send: `require(${JSON.stringify(moduleFilename)}).number`,
     expect: '42'
-  }
+  },
+  {
+    send: 'import comeOn from \'fhqwhgads\'',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      'SyntaxError: Cannot use import statement inside the Node.js REPL, \
+alternatively use dynamic import: const { default: comeOn } = await import("fhqwhgads");',
+    ]
+  },
+  {
+    send: 'import { export1, export2 } from "module-name"',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      'SyntaxError: Cannot use import statement inside the Node.js REPL, \
+alternatively use dynamic import: const { export1, export2 } = await import("module-name");',
+    ]
+  },
+  {
+    send: 'import * as name from "module-name";',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      'SyntaxError: Cannot use import statement inside the Node.js REPL, \
+alternatively use dynamic import: const name = await import("module-name");',
+    ]
+  },
+  {
+    send: 'import "module-name";',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      'SyntaxError: Cannot use import statement inside the Node.js REPL, \
+alternatively use dynamic import: await import("module-name");',
+    ]
+  },
+  {
+    send: 'import { export1 as localName1, export2 } from "bar";',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      'SyntaxError: Cannot use import statement inside the Node.js REPL, \
+alternatively use dynamic import: const { export1: localName1, export2 } = await import("bar");',
+    ]
+  },
+  {
+    send: 'import alias from "bar";',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      'SyntaxError: Cannot use import statement inside the Node.js REPL, \
+alternatively use dynamic import: const { default: alias } = await import("bar");',
+    ]
+  },
+  {
+    send: 'import alias, {namedExport} from "bar";',
+    expect: [
+      kSource,
+      kArrow,
+      '',
+      'Uncaught:',
+      'SyntaxError: Cannot use import statement inside the Node.js REPL, \
+alternatively use dynamic import: const { default: alias, namedExport } = await import("bar");',
+    ]
+  },
 ];
 
 (async function() {
@@ -765,8 +907,9 @@ const tcpTests = [
 
     socket.end();
   }
-  common.allowGlobals(...Object.values(global));
-})();
+  common.allowGlobals(global.invoke_me, global.message, global.a, global.blah,
+                      global.I, global.f, global.path, global.x, global.name, global.foo);
+})().then(common.mustCall());
 
 function startTCPRepl() {
   let resolveSocket, resolveReplServer;
@@ -800,7 +943,7 @@ function startTCPRepl() {
 
   return Promise.all([
     new Promise((resolve) => resolveSocket = resolve),
-    new Promise((resolve) => resolveReplServer = resolve)
+    new Promise((resolve) => resolveReplServer = resolve),
   ]);
 }
 
@@ -845,7 +988,7 @@ function startUnixRepl() {
 
   return Promise.all([
     new Promise((resolve) => resolveSocket = resolve),
-    new Promise((resolve) => resolveReplServer = resolve)
+    new Promise((resolve) => resolveReplServer = resolve),
   ]);
 }
 
@@ -855,7 +998,7 @@ function event(ee, expected) {
       const data = inspect(expected, { compact: false });
       const msg = `The REPL did not reply as expected for:\n\n${data}`;
       reject(new Error(msg));
-    }, common.platformTimeout(500));
+    }, common.platformTimeout(9999));
     ee.once('data', common.mustCall((...args) => {
       clearTimeout(timeout);
       resolve(...args);
